; -----------------------------------------------------------------------------
; USB Flash Memory Writer Firmware
;  programmed by M.yanagisawa.(MoroboshiLUM)  All rights reserved.
; for ATMEL AT90S4414 8bit RISC CPU & OKI MSM60851 USB Controler
; -----------------------------------------------------------------------------

.include "4414def.inc"

; Generic Register difination

; r0 is used by LPM instruction
.def	accmode = r2		; access mode
.def	one	= r3
.def	zero	= r4
.def	adr0	= r5
.def	sum1	= r6
.def	sum2	= r7
.def	sum3	= r8
.def	sum0	= r9
.def	adr1	= r10
.def	adr2	= r11
.def	adr3	= r12
.def	inttemp3= r13		; using temporary regs while INTR
.def	adrstop = r14		; address stopper
.def	pos	= r15		; disp pos
; ---
.def	inttemp	= r16		; using temporary regs while INTR
.def	inttemp2= r17		; using temporary regs while INTR
.def	temp	= r18		; broken regs when any called.
.def	stack	= r19		; 
.def	wparam  = r20		; parameter
.def	status	= r21		; executing status
.def	temp2	= r22

.equ	DISPOFS	= 64

.equ	CHECKSUM_MSG	= 0x80
.equ	WRITE_MSG	= 0x81
.equ	BLANK_MSG	= 0x82
.equ	CHIPERASE_MSG	= 0x83
.equ	READY_MSG	= 0x84
.equ	READ_MSG	= 0x85
;
; USB Standard Requests
; 
.equ	GET_STATUS	= 0
.equ	CLEAR_FEATURE	= 1
.equ	SET_FEATURE	= 3
.equ	SET_ADDRESS	= 5
.equ	GET_DESCRIPTOR	= 6
.equ	SET_DESCRIPTER	= 7
.equ	GET_CONFIGRATION = 8
.equ	SET_CONFIGRATION = 9
.equ	GET_INTERFACE	= 10
.equ	SET_INTERFACE	= 11
.equ	SYNC_FRAME	= 12

;; MSM60851 Registers
.equ	PKTRDY_W	= 0x48
.equ	PKTRDY_R	= 0xc8
.equ	EP0STAT_R	= 0xf3
.equ	EP0STAT_W	= 0x73
.equ	INTSTAT		= 0xdc
.equ	INTENBL_R	= 0xdb
.equ	INTENBL_W	= 0x5b
.equ	DVCADR_R	= 0xc0
.equ	DVCADR_W	= 0x40
.equ	EP0RXCNT	= 0xc9
.equ	EP1RXCNT	= 0xca
.equ	EP0RXFIFO	= 0x40
.equ	EP1RXFIFO	= 0x41
.equ	EP0TXPLD_R	= 0xf2
.equ	EP0TXPLD_W	= 0x72
.equ	EP0TXFIFO	= 0xc0

.equ	EP1CON_R	= 0xe4
.equ	EP1CON_W	= 0x64
.equ	EP1TGL_R	= 0xe5
.equ	EP1TGL_W	= 0x65
.equ	EP1PLD_R	= 0xe6
.equ	EP1PLD_W	= 0x66

.equ	EP2CON_R	= 0xf4
.equ	EP2CON_W	= 0x74
.equ	EP2TGL_R	= 0xf5
.equ	EP2TGL_W	= 0x75
.equ	EP2PLD_R	= 0xf6
.equ	EP2PLD_W	= 0x76
.equ	EP2TXFIFO	= 0xc2
.equ	CLRFIFO		= 0x4e

.equ	DEV_DEVICE	= 1
.equ	DEV_CONFIGRATION = 2
.equ	DEV_STRING	= 3
.equ	DEV_INTERFACE	= 4
.equ	DEV_ENDPOINT	= 5

.equ	TYPE_OF_STANDARD = 0x00
.equ	TYPE_OF_CLASS	= 0x20
.equ	TYPE_OF_VENDOR	= 0x40

.equ	SETUP_RDY_INTR	= 0x01
.equ	EP1_RDY_INTR	= 0x02
.equ	EP2_RDY_INTR	= 0x04
.equ	EP0_RECV_INTR	= 0x08
.equ	EP0_XMIT_INTR	= 0x10
.equ	BUS_RESET_INTR	= 0x20
.equ	SUSPEND_INTR	= 0x40
.equ	EP3_RDY_INTR	= 0x80

.equ	DEVICE_RECIPIENT = 0x00
.equ	INTERFACE_RECICPENT = 0x01
.equ	ENDPOINT_RECICPENT = 0x02

.equ	DEVICEtoHOST	= 0x80
.equ	DATA_SET = 1
.equ	DATA_RESET = 0

.equ	_STATUS_OF_DISPSTR	= 1
.equ	_STATUS_OF_CHECKSUM	= 5
.equ	_STATUS_OF_BLANK 	= 6
.equ	_STATUS_OF_CHIPERASE	= 7
.equ	_STATUS_OF_SECTORERASE	= 8
.equ	_STATUS_OF_READ		= 9
.equ	_STATUS_OF_WRITE	= 10

	.CSEG
	.ORG	0

; reset vector

	rjmp	RESET		; Reset handle
	rjmp	INT0_VECT	; Ext int0
	rjmp	RESET		; Ext int1
	rjmp	RESET		; Timer1 capture handler
	rjmp	RESET		; Timer1 compA handler
	rjmp	RESET		; Timer1 compB handler
	rjmp	RESET		; Timer1 ov1 handler
	rjmp	RESET		; Timer0 ov1 handler
	rjmp	RESET		; spi handler
	rjmp	RESET		; uart rx handler
	rjmp	RESET		; uart dre handler
	rjmp	RESET		; uart txc handler
	rjmp	RESET		; anacomp handler

;
; WAIT 4mili sec
;
; 6.14MHz @ 1clock is 163ns
wait4m:
	ldi	temp,32	; 765 * 32 + 6*32 = 24672 * 163ns = 4.01ms
wait4m_loop0:
	push	temp		; +1
	ldi	temp,255	; +1 3*255=765 Clock
wait4m_loop1:
	dec	temp		; +1
	brne	wait4m_loop1	; +2 (through 1clock)
	pop	temp		; +1
	dec	temp		; +1
	brne	wait4m_loop0	; +2

	ret

wait40us:
	ldi	temp,82			; 3*82*163ns = 40usec wait
wait40u_loop:
	dec	temp
	brne	wait40u_loop

	ret

;
; Incriment Column (for Hitachi LCDC HD)
;
incpos:
	mov	temp,pos
	inc	temp
	cpi	temp,DISPOFS
	brcs	incpos_01
	cpi	temp,DISPOFS+8
	brcs	incpos_exit
	clr	pos
	ret
incpos_01:
	cpi	temp,8
	brcs	incpos_exit
	ldi	temp,DISPOFS
incpos_exit:
	mov	pos,temp

	ret

; 8 bit data bus writing.
; wparam : writereg 4bit
writecmd:
;; PD ok;
	ldi	ZH,0x20			; RS=0
	st	Z,wparam
	
	rcall	wait40us

	ret

; 8 bit data bus writing.
; wparam : writereg 4bit
writedata:
;; PD ok;
	ldi	ZH,0x30			; RS=1
	st	Z,wparam

	rcall	wait40us

	ret
;
; ----------------------------------------
; chr <- ChrCode
; ----------------------------------------
PutChr:
	push	wparam
	mov	wparam,pos
	sbr	wparam,0x80
	rcall	writecmd
	pop	wparam
	rcall	writedata

	ret
;
; Binary to HEX
; in  temp
; out SRAM(hexl & hexh)
BinToHex:
	push	temp
	andi	temp,0x0f
	subi	temp,-0x30
	cpi	temp,0x3a
	brcs	next_01
	; if temp>0x39
	subi	temp,-$7
next_01:
	sts	hexl,temp
	pop	temp
	lsr	temp
	lsr	temp
	lsr	temp
	lsr	temp
;; high
	subi	temp,-$30
	cpi	temp,0x3a
	brcs	next_02
	; if temp>0x39 
	subi	temp,-$7
next_02:
	sts	hexh,temp

	ret

; -----------------------------------------------
; Initialize LCD
; -----------------------------------------------
InitLCD:
    ; WAIT 16msec
    	cbi	PORTD,4		;;	power on

	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	ldi	wparam,0x30
	rcall	writecmd
	rcall	wait4m
	ldi	wparam,0x30
	rcall	writecmd
;;
	ldi	temp,25
wait100msec:
	push	temp
	rcall	wait4m
	pop	temp
	dec	temp
	brne	wait100msec
;;
	ldi	wparam,0x30
	rcall	writecmd
;;
;;
;; -------- initialize ok. -----------------
;;
	ldi	wparam,0x3c	; DL=1,N=1,F=0
	rcall	writecmd
;; disp off
	ldi	wparam,0x08
	rcall	writecmd
;; clear display
	ldi	wparam,0x01
	rcall	writecmd
	rcall	wait4m
;; Entry Mode
	ldi	wparam,0x06
	rcall	writecmd
;; display cursor off
	ldi	wparam,0x0c
	rcall	writecmd
;;
	ret

; ------------------------
; disp string
; ------------------------
dispstr:
	push	wparam
	clr	temp2
dispstr_loop:
	lpm
	adiw	ZL,1
	tst	r0
	breq	dispend
	inc	temp2
	mov	wparam,r0
	push	ZH
	rcall	putchr
	pop	ZH
	rcall	incpos
	rjmp	dispstr_loop
dispend:
	pop	wparam
	tst	wparam
	brne	PC+2
	ret
	ldi	temp,16
	sub	temp,temp2
	tst	temp
	breq	dispexit
	mov	temp2,temp
dispspace:
	push	ZH
	ldi	wparam,' '
	rcall	putchr
	pop	ZH
	rcall	incpos
	dec	temp2
	brne	dispspace
dispexit:
	ret

; ----------------------------------------------------
; USB-Chip ENABLE & initialize
; with wait 4milli second
; ----------------------------------------------------
EnableUSB:
;; Hardware reset
	ldi	ZH,0x80
	ldi	ZL,0x5a		; POLSEL
	clr	temp
	st	Z+,temp		; POLSEL=0x00
	ldi	temp,BUS_RESET_INTR | SETUP_RDY_INTR
	st	Z+,temp		; INTENBL_W=0x21
	adiw	ZL,1		;
	clr	temp
	st	Z,temp		; DMACON=0x00
	ldi	ZL,EP0TXPLD_W
	ldi	temp,0x08	; EP0 Transmit Payload size
	st	Z,temp		; Number of FIFO size is 8 for MSM60851B 
	ldi	ZL,EP0STAT_R
	ld	temp,Z
	cbr	temp,0x04
	st	Z,temp

;; initialize statuses
	ser	temp
	sts	usbconfig,temp
	clr	temp
	sts	remotewakeup,temp
	sts	usbaddress,temp
	sts	EP0xmitsize,temp
	sts	EP0recvsize,temp

;	ser	temp
;	sts	prevchr,temp
;	sts	prevmark,temp
;	clr	temp
;	sts	dispchr,temp
;	ldi	temp,' '
;	sts	dispmark,temp
	ser	temp
	sts	prevmark,temp
	sts	dispmark,temp
;; debug
;;	clr	r23
;;	clr	r24
;;
	clr	status
	clr	one
	inc	one
	clr	zero

	ret

; -----------------------------------------------------
; RESET VECTOR
; -----------------------------------------------------
RESET:
;; initializing ports
	cli				; disable interrupt

	ldi	temp,high(RAMEND)
	out	SPH,temp
	ldi	temp,low(RAMEND)
	out	SPL,temp

; internal sram all NULL Clear!
	ldi	YL,0x60
	clr	YH
	clr	temp
	clr	inttemp
reset_clearloop:
	st	Y+,temp
	dec	inttemp
	brne	reset_clearloop

	;; interrupt 0 enable
	ldi	temp,(1<<INT0)
	out	GIMSK,temp	

	;; sram interface enable
	ldi	temp,(1<<SRE)
	out	MCUCR,temp

	; output port a,b,c
	ldi	temp,0xF3	; AIN0,1 is  input
	out	DDRB,temp
	ldi	temp,0xFF
	out	DDRA,temp	; for SRAM IF
	out	DDRC,temp	; for SRAM IF

	; output port d
	ldi	temp,0xFB	;; B is input 
	out	DDRD,temp

	; disable USB
	ldi	temp,0x00
	out	PORTB,temp
	out	ACSR,temp	;; compareter enable

	; disable output
	ldi	temp,0x30	;; disable Flash,LCD i/f disable
	out	PORTD,temp

	;; Y register is used for accessing USB Controler ONLY
	ldi	YH,0x80

	rcall	clearadrs

; hardware reset
	sbi	PORTB,0		;
	rcall	wait40us
	cbi	PORTB,0		; hardware reset USB
	rcall	wait40us	; low while 40us
;; 
	rcall	InitLCD		;; initialize lcd

	sbi	PORTB,0		; enable USB

	rcall	EnableUSB	;;

	sei

	clr	pos
	ldi	ZL,low(2*initmsg)
	ldi	ZH,high(2*initmsg)
	ser	wparam
	rcall	dispstr
	
; =========================================================================
mainloop:
	sei
	nop	; Enable Interrupt
	nop	;
	nop	;
	nop	;
	nop	;
;; Message Display code (for debug)
;	cli
;	lds	temp,prevchr
;	lds	wparam,dispchr
;	sei
;	cp	temp,wparam	;; wait while changing
;	breq	checkmark
;	push	wparam
;	ldi	temp,DISPOFS+1
;	mov	pos,temp
;	mov	temp,wparam
;	rcall	BinToHex
;	lds	wparam,hexh
;	rcall	putchr
;	rcall	incpos
;	lds	wparam,hexl
;	rcall	putchr
;	pop	wparam
;	sts	prevchr,wparam
checkmark:
	cli
	lds	temp,prevmark
	lds	wparam,dispmark
;;	mov	temp,r23
;;	mov	wparam,r24
	sei
	cp	temp,wparam	;; wait while changing
	breq	main_execution
	push	wparam
;;;	rjmp	dispmark_normal	;;; for debug
	ldi	ZL,low(readingmsg*2)
	ldi	ZH,high(readingmsg*2)
	cpi	wparam,READ_MSG
	brne	PC+2
	rjmp	dispmessage
	ldi	ZL,low(chiperasemsg*2)
	ldi	ZH,high(chiperasemsg*2)
	cpi	wparam,CHIPERASE_MSG
	brne	PC+2
	rjmp	dispmessage
	ldi	ZL,low(blankchecking*2)
	ldi	ZH,high(blankchecking*2)
	cpi	wparam,BLANK_MSG
	brne	PC+2
	rjmp	dispmessage
	ldi	ZL,low(writingmsg*2)
	ldi	ZH,high(writingmsg*2)
	cpi	wparam,WRITE_MSG
	brne	PC+2
	rjmp	dispmessage
	ldi	ZL,low(chksummsg*2)
	ldi	ZH,high(chksummsg*2)
	cpi	wparam,CHECKSUM_MSG
	brne	PC+2
	rjmp	dispmessage
	ldi	ZL,low(readymsg*2)
	ldi	ZH,high(readymsg*2)
dispmessage:
	clr	pos
	ser	wparam
	rcall	dispstr
	rjmp	checkmark_exit
dispmark_normal:
	ldi	temp,DISPOFS+4
	mov	pos,temp
	rcall	putchr
;	rcall	incpos
checkmark_exit:
	pop	wparam
	sts	prevmark,wparam
;;	mov	r23,wparam

;; -----------------------------------------------------------------------
;; Execution
;;	sleep	;; Cpu Enters SLEEP Mode(wait while not Intr requested)
main_execution:
	cli
	lds	temp,prevmark
	lds	wparam,dispmark
	cp	temp,wparam	;; if different mark then loop
	brne	mainloop
	mov	temp,status	;; check routine
	sbrc	temp,6		;; if not EP0 status sent,i ignore status
	rjmp	mainloop
	sbrc	temp,7		;; already fin. no check
	rjmp	mainloop
;; check frame
	andi	temp,0x3f
	cpi	temp,_STATUS_OF_DISPSTR
	brne	PC+2
	rjmp	Main_DispStr
	cpi	temp,_STATUS_OF_CHECKSUM
	brne	PC+2
	rjmp	Main_CalcChecksum
	cpi	temp,_STATUS_OF_BLANK
	brne	PC+2
	rjmp	Main_BlankCheck
	sei
	rjmp	mainloop

; =============================================================================
; ----------------------------------------------
; Display String
; ----------------------------------------------
Main_DispStr:
	sei
	ldi	temp2,16		; dispsize
	ldi	ZL,low(vendorstr)
	ldi	ZH,high(vendorstr)
	clr	pos			; position clear
checkvstr_loop:
	ld	wparam,Z+
	push	ZH
	rcall	putchr
	rcall	incpos
	pop	ZH
	dec	temp2
	brne	checkvstr_loop
checkvstr_fin:
	clr	status			; fin execution
	rjmp	mainloop

; ----------------------------------------------
; Calcuulating Check sum.
; ----------------------------------------------
Main_CalcChecksum:
	sei
	rcall	clearadrs
chksum_loop0:
	ldi	ZH,0x02
	clr	ZL
chksum_loop1:
	ld	temp,Z+
	add	sum0,temp
	adc	sum1,zero
	adc	sum2,zero
	adc	sum3,zero
	tst	ZL
	brne	chksum_loop1
;
	add	adr1,one
	adc	adr2,zero
	ldi	ZH,0x04
	mov	ZL,adr1		; address load
	ld	temp,Z
	ldi	ZH,0x08
	mov	ZL,adr2
	ld	temp,Z		; address load
	cp	ZL,adrstop
	brne	chksum_loop0

	sbr	status,0x80	; fin
	ldi	temp,READY_MSG
	sts	dispmark,temp
	rjmp	mainloop

; ----------------------------------------------
; Blank Check
; ----------------------------------------------
Main_BlankCheck:
	sei
	rcall	clearadrs
	clr	sum0
blank_loop0:
	ldi	ZH,0x02
	clr	ZL
blank_loop1:
	ld	temp,Z+
	cpi	temp,0xFF
	brne	blank_err
	tst	ZL
	brne	blank_loop1
;
	add	adr1,one
	adc	adr2,zero
	ldi	ZH,0x04
	mov	ZL,adr1		; address load
	ld	temp,Z
	ldi	ZH,0x08
	mov	ZL,adr2
	ld	temp,Z		; address load
	cp	ZL,adrstop
	brne	blank_loop0
	rjmp	blank_fin
blank_err:
	com	sum0		; err set.
blank_fin:
	sbr	status,0x80	; fin
	ldi	temp,READY_MSG
	sts	dispmark,temp
	rjmp	mainloop

; =============================================================================
; ----------------------------------------------
; USB Bus Reset Request
; #Status Clear only.
; ----------------------------------------------
reqUSBBusReset:
	ldi	inttemp,0x20	;; USB Bus reset Clear
	ldi	YL,0x41		;; Write DVCSTAT
	st	Y,inttemp	;; Clear Bus Reset

	push	temp
	rcall	EnableUSB	;; Reset Variables
	pop	temp

	rjmp	reqReturn

; -----------------------------------------------
; Execute Vendor Requests
; inttemp : Request Number.
; -----------------------------------------------
ExecuteVendorReq:
	cpi	inttemp,0
	brne	PC+2
	rjmp	VR_Initialize
	cpi	inttemp,1
	brne	PC+2
	rjmp	VR_DisplayString
	cpi	inttemp,2
	brne	PC+2
	rjmp	VR_ROMPowerOnOff
	cpi	inttemp,3
	brne	PC+2
	rjmp	VR_GetSignature
	cpi	inttemp,4
	brne	PC+2
	rjmp	VR_SetAddress
	cpi	inttemp,5
	brne	PC+2
	rjmp	VR_CheckSum
	cpi	inttemp,6
	brne	PC+2
	rjmp	VR_Blank
	cpi	inttemp,7
	brne	PC+2
	rjmp	VR_ChipErase
	cpi	inttemp,8
	brne	PC+2
	rjmp	VR_SectorErase
	cpi	inttemp,9
	brne	PC+2
	rjmp	VR_Read
	cpi	inttemp,10
	brne	PC+2
	rjmp	VR_Write
	cpi	inttemp,11
	brne	PC+2
	rjmp	VR_Enquire

NotSupportReq:
	ldi	YL,EP0STAT_W
	ldi	inttemp,0x04
	st	Y,inttemp		;; set
	clr	inttemp
	sts	EP0recved,inttemp
	rjmp	Exec_FuncReturn

; -------------------------------------------------------
;  Address Latch Clear & init variables
; -------------------------------------------------------
clearadrs:
	ldi	ZH,0x0e
	clr	ZL
	ld	sum0,Z		; Address latch clear
	clr	sum0
	clr	sum1
	clr	sum2
	clr	sum3
	clr	adr0
	clr	adr1
	clr	adr2
	clr	adr3

	ret

; ------------------------------------------------------
; Initialize
; Request is 0
; ------------------------------------------------------
VR_Initialize:
	sbis	PORTD,3		;; ROM Power check
	;; not executing
	rjmp	VR_init_vclear
	;; executing
	ldi	ZH,0x02
	ldi	inttemp,0xf0
	st	Z,inttemp	;; flash reset.
	;;
	sbi	PORTD,5		; Hi-Z
	cbi	PORTD,3		; POWER OFF
	ldi	inttemp,20
VR_Init_wait:
	dec	inttemp
	brne	VR_Init_wait
VR_init_vclear:
	;; USB Intr clear
	ldi	YL,INTENBL_W
	ldi	inttemp,BUS_RESET_INTR | SETUP_RDY_INTR
	st	Y,inttemp
	;; rx EP1,tx EP2 Clear
	ldi	YL,CLRFIFO
	ldi	inttemp,0x04	;; EP2 Clear
	st	Y,inttemp
;;
	clr	status
	clr	adrstop
	rcall	clearadrs
	ldi	inttemp,READY_MSG
	sts	dispmark,inttemp
	rjmp	Exec_FuncReturn

; ------------------------------------------------------
; Display String on LCD
; Request is 1
; ------------------------------------------------------
VR_DisplayString:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall
	ldi	XL,low(vendorstr)
	ldi	XH,high(vendorstr)
; clear buf
	ldi	inttemp2,' '
	ldi	inttemp,16
setspace_loop:
	st	X+,inttemp2
	dec	inttemp
	brne	setspace_loop
	st	X,zero
;
	ldi	ZL,low(EP0recvbuf)
	ldi	ZH,high(EP0recvbuf)
	ldi	XL,low(vendorstr)
	ldi	XH,high(vendorstr)
	lds	inttemp,EP0recvsize
	cpi	inttemp,16
	brcs	PC+2
	ldi	inttemp,16
resetlength_vstr:
	mov	inttemp2,inttemp
	tst	inttemp2
	breq	setterm_vstr
copyvstr:
	ld	inttemp,Z+
	st	X+,inttemp
	dec	inttemp2
	brne	copyvstr
setterm_vstr:
	ldi	status,_STATUS_OF_DISPSTR
	sts	EP0recved,zero
	rjmp	VR_ReqNoXmitReturn	; not sent.(size=0)

; --------------------------------------------------------
; Rom Power On / Off
; Request is 2
; Value = 0 OFF
; --------------------------------------------------------
VR_ROMPowerOnOff:
	ldi 	inttemp,20
	lds	inttemp2,devwValueL
	tst	inttemp2
	brne	VR_RPW_ON
;; OFF
	sbi	PORTD,5		; Hi-Z
	cbi	PORTD,3		; POWER OFF
	rjmp	VR_RPW_Ok
VR_RPW_ON:
	sbi	PORTD,3		; POWER ON
	cbi	PORTD,5		; Link.
VR_RPW_Ok:
	dec	inttemp		; Wait 10us.
	brne	VR_RPW_Ok
	rjmp	Exec_FuncReturn

; --------------------------------------------------------
; Get Signature
; Request is 3
; --------------------------------------------------------
VR_GetSignature:
	tst	status		; check executing
	breq	PC+2
	rjmp	Exec_FuncStall
	sbis	PORTD,3
	rjmp	Exec_FuncStall
; Reset
	ldi	XH,high(EP0xmitbuf)
	ldi	XL,low(EP0xmitbuf)
	rcall	SendResetBCmd	; reset B (for ATMEL)
	push	temp		;; 20msec wait
	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	pop	temp
;;
	ldi	ZH,0x0e
	clr	ZL
	ld	inttemp,Z	;; clear latch.
;;
	ldi	ZH,0x06		;
	clr	ZL
	ld	inttemp,Z+	; +0
	ldi	ZH,0x02
	ld	inttemp2,Z+	; +1
	ld	inttemp3,Z	; +2
	st	X+,inttemp	; original
	st	X+,inttemp2	; original
	st	X+,inttemp3	; original
; signature cmd A
	ldi	ZH,0x06
	ldi	ZL,0xAA		;; Address = 00AAAA
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = 005555
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = 00AAAA
	ldi	inttemp,0x90	;; Data = 90
	st	Z,inttemp
	clr	ZL
	ld	inttemp,Z	; A0=0 A1=0 Address=xx0000
	ldi	ZH,0x02		; latch0 select only
	ldi	ZL,0x02
	ld	inttemp2,Z	; A0=0 A1=1
	st	X+,inttemp	; cmd A
	st	X+,inttemp2	; cmd A
;; reset
	ldi	ZH,0x06
	ldi	ZL,0xAA		;; Address = 002AAA
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
; signature cmd B
; for ATMEL and 8bit code
	ldi	ZH,0x06
	ldi	ZL,0x55		;; Address = 005555
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZH,0x04
	ldi	ZL,0x2A
	ld	inttemp,Z
	ldi	ZH,0x02
	ldi	ZL,0xAA		;; Address = 002AAA
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZH,0x06
	ldi	ZL,0x55		;; Address = 005555
	ldi	inttemp,0x90	;; Data = 90
	st	Z,inttemp
	push	temp		;; 20msec wait
	rcall	wait40us
	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	rcall	wait4m
	pop	temp
	clr	ZL		; 
	ld	inttemp,Z	; Address=000000
	ldi	ZH,0x02		; latch0 select only
	ldi	ZL,0x01
	ld	inttemp2,Z	; Address=000001
	st	X+,inttemp	; cmd B
	st	X+,inttemp2	; cmd B
;; reset cmd
	rcall	SendResetBCmd
;;
	ldi	ZH,0x08
	mov	ZL,adr2
	ld	inttemp,Z	;; latch restore.
;; xmit set.
	ldi	inttemp,7	;; 
	rjmp	VR_ReqXmitReturn

; --------------------------------------------------
;  Send Reset cmd for 8bit or ATMEL
; --------------------------------------------------
SendResetBCmd:
	ldi	ZH,0x08
	clr	ZL
	ld	inttemp,Z	;; 
	ldi	ZH,0x06
	ldi	ZL,0x55		;; Address = 005555
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZH,0x04
	ldi	ZL,0x2A
	ld	inttemp,Z
	ldi	ZH,0x02
	ldi	ZL,0xAA		;; Address = 002AAA
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZH,0x06
	ldi	ZL,0x55		;; Address = 005555
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
	ldi	ZH,0x08
	mov	ZL,adr2
	ld	inttemp,Z	;; reset adr2
	ret

; --------------------------------------------------------
; Set Address Stop
; Request is 4
; --------------------------------------------------------
VR_SetAddress:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall		; now executing ?
	lds	adrstop,devwValueH	; hi address load
	rjmp	VR_ReqNoXmitReturn

; --------------------------------------------------------
; Calculation Check sum.
; Request is 5
; --------------------------------------------------------
VR_CheckSum:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall		;; now executing?
	tst	adrstop
	brne	PC+2
	rjmp	Exec_FuncStall		;; not set address stopper?
	sbis	PORTD,3			;; rom power on ?
	rjmp	Exec_FuncStall
	ldi	status,_STATUS_OF_CHECKSUM	; MSB bit is fin bit.
	ldi	inttemp,CHECKSUM_MSG
	sts	dispmark,inttemp
	rjmp	VR_ReqNoXmitReturn

; --------------------------------------------------------
; Checking Blank
; Request is 6
; --------------------------------------------------------
VR_Blank:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall		;; now executing?
	tst	adrstop
	brne	PC+2
	rjmp	Exec_FuncStall		;; not set address stopper?
	sbis	PORTD,3			;; rom power on ?
	rjmp	Exec_FuncStall
	ldi	status,_STATUS_OF_BLANK		; MSB bit is fin bit.
	ldi	inttemp,BLANK_MSG
	sts	dispmark,inttemp
	rjmp	VR_ReqNoXmitReturn

; --------------------------------------------------------
; Chip Erase
; Request is 7
; --------------------------------------------------------
VR_ChipErase:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall		;; now executing?
	sbis	PORTD,3			;; rom power on ?
	rjmp	Exec_FuncStall
	ldi	status,_STATUS_OF_CHIPERASE		; MSB bit is fin bit.

	ldi	inttemp,CHIPERASE_MSG
	sts	dispmark,inttemp

	lds	inttemp,devwValueL	;; Erase Cmd Type
	tst	inttemp
	breq	Req_chiperaseA
	rcall	Flash_ChipErase_B
	rjmp	VR_ReqNoXmitReturn

Req_chiperaseA:
	rcall	Flash_ChipErase_A
	rjmp	VR_ReqNoXmitReturn

; --------------------------------------------------------
; Sector Erase
; Request is 8
; --------------------------------------------------------
VR_SectorErase:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall		;; now executing?
	sbis	PORTD,3			;; rom power on ?
	rjmp	Exec_FuncStall
	ldi	status,_STATUS_OF_SECTORERASE		; MSB bit is fin bit.
	lds	inttemp2,devwIndexL	;; set sector address(L)
	lds	inttemp3,devwIndexH	;; set sector address(H)
	lds	inttemp,devwValueL	;; Erase Cmd Type
	tst	inttemp
	breq	Req_sectoreraseA
	rcall	Flash_SectorErase_B
	rjmp	VR_ReqNoXmitReturn

Req_sectoreraseA:
	rcall	Flash_SectorErase_A
	rjmp	VR_ReqNoXmitReturn

; --------------------------------------------------------
; Read
; Request is 9
; --------------------------------------------------------
VR_Read:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall		;; now executing?
	tst	adrstop
	brne	PC+2
	rjmp	Exec_FuncStall		;; not set address stopper?
	sbis	PORTD,3			;; rom power on ?
	rjmp	Exec_FuncStall

	rcall	clearadrs
	ldi	status,_STATUS_OF_READ	; MSB bit is fin bit.
	ldi	inttemp,READ_MSG
	sts	dispmark,inttemp

	ldi	YL,CLRFIFO
	ldi	inttemp,0x04	;; EP2 Clear
	st	Y,inttemp
	ldi	YL,INTENBL_R
	ld	inttemp,Y
	sbr	inttemp,EP2_RDY_INTR
	ldi	YL,INTENBL_W		;; Enable EP2 TX INTR
	st	Y,inttemp
	rjmp	VR_ReqNoXmitReturn

; --------------------------------------------------------
; Write
; Request is 10
; --------------------------------------------------------
VR_Write:
	tst	status
	breq	PC+2
	rjmp	Exec_FuncStall		;; now executing?
	tst	adrstop
	brne	PC+2
	rjmp	Exec_FuncStall		;; not set address stopper?
	sbis	PORTD,3			;; rom power on ?
	rjmp	Exec_FuncStall

	rcall	clearadrs
	ldi	status,_STATUS_OF_WRITE	; MSB bit is fin bit.
	lds	accmode,devwValueL	; Write Cmd Type
	ldi	inttemp,WRITE_MSG
	sts	dispmark,inttemp

	ldi	YL,INTENBL_R
	ld	inttemp,Y
	sbr	inttemp,EP1_RDY_INTR
	ldi	YL,INTENBL_W		; Enable EP1 RX INTR
	st	Y,inttemp
	rjmp	VR_ReqNoXmitReturn

; --------------------------------------------------------
; Enquire
; Request is 11
; --------------------------------------------------------
VR_Enquire:
	ldi	XH,high(EP0xmitbuf)
	ldi	XL,low(EP0xmitbuf)
	sbrc	status,6
	rjmp	Exec_FuncStall		;; if executing xmitting EP0 then stall.
;; return data set.
;; function was fin.
	mov	inttemp,status
	andi	inttemp,0x3f		;;
	cpi	inttemp,_STATUS_OF_CHECKSUM
	brne	PC+2
	rjmp	VR_ReqCheckSum
	cpi	inttemp,_STATUS_OF_BLANK
	brne	PC+2
	rjmp	VR_ReqBlankResult
	cpi	inttemp,_STATUS_OF_CHIPERASE
	brne	PC+2
	rjmp	VR_CheckToggle
	cpi	inttemp,_STATUS_OF_SECTORERASE
	brne	PC+2
	rjmp	VR_CheckToggle
	cpi	inttemp,_STATUS_OF_WRITE
	brne	PC+2
	rjmp	VR_ReqWrite
	cpi	inttemp,_STATUS_OF_READ
	brne	PC+2
	rjmp	VR_ReqRead

VR_NormalReturn:
	st	X+,status
VR_NormalReturn_notstat:
	in	inttemp,ACSR
	st	X+,inttemp
	st	X+,zero
	st	X+,zero
	st	X+,zero
	st	X+,zero
	ldi	inttemp,6		;; status only.
	rjmp	VR_ReqXmitReturn

; -------------------------------------------------
; Enquire Toggle bit status
; -------------------------------------------------
VR_CheckToggle:
	rcall	Flash_CheckToggle
	brcs	Flash_Exec_NG
	breq	Flash_Exec_Ok

; not fin
	st	X+,status
	in	inttemp,ACSR
	st	X+,inttemp
	st	X+,zero
	st	X+,zero
	st	X+,zero
	st	X+,zero
	ldi	inttemp,6
	rjmp	VR_ReqXmitReturn

; timeover.
Flash_Exec_NG:
	lds	inttemp,dispmark
	cpi	inttemp,CHIPERASE_MSG
	brne	Flash_Exec_NG_2
	ldi	inttemp,READY_MSG
	sts	dispmark,inttemp
Flash_Exec_NG_2:
	sbr	status,0x80
	st	X+,status
	in	inttemp,ACSR
	st	X+,inttemp
	st	X+,one			;; NG.
	st	X+,zero
	st	X+,zero
	st	X+,zero
	ldi	inttemp,6
	clr	status
	rjmp	VR_ReqXmitReturn

; fin
Flash_Exec_Ok:
	lds	inttemp,dispmark
	cpi	inttemp,CHIPERASE_MSG
	brne	Flash_Exec_OK_2
	ldi	inttemp,READY_MSG
	sts	dispmark,inttemp
Flash_Exec_OK_2:
	sbr	status,0x80
	st	X+,status
	in	inttemp,ACSR
	st	X+,inttemp
	st	X+,zero			;; OK.
	st	X+,zero
	st	X+,zero
	st	X+,zero
	ldi	inttemp,6
	clr	status
	rjmp	VR_ReqXmitReturn

; -------------------------------------------------
; Blank Result
; -------------------------------------------------
VR_ReqBlankResult:
	sbrs	status,7
	rjmp	VR_NormalReturn
	st	X+,status
	in	inttemp,ACSR
	st	X+,inttemp
	st	X+,sum0			;; sum0 is blank result.
	st	X+,zero		; dummy
	st	X+,zero		; dummy
	st	X+,zero		; dummy
	clr	status
	ldi	inttemp,6
	rjmp	VR_ReqXmitReturn

; -------------------------------------------------
; checksum
; -------------------------------------------------
VR_ReqCheckSum:
	sbrs	status,7
	rjmp	VR_NormalReturn
	st	X+,status
	in	inttemp,ACSR
	st	X+,inttemp
	st	X+,sum0
	st	X+,sum1
	st	X+,sum2
	st	X+,sum3
	ldi	inttemp,6		;; check sum.
	clr	status
	rjmp	VR_ReqXmitReturn	;; 

; -------------------------------------------------
; Write
; -------------------------------------------------
VR_ReqWrite:
	sbrs	status,7
	rjmp	VR_NormalReturn
	ldi	inttemp,READY_MSG
	sts	dispmark,inttemp
	ldi	YL,INTENBL_R
	ld	inttemp,Y
	cbr	inttemp,EP1_RDY_INTR	;; DISABLE EP1
	ldi	YL,INTENBL_W
	st	Y,inttemp
	st	X+,status		;; store status
	clr	status			;; status clear
	rjmp	VR_NormalReturn_notstat

; -------------------------------------------------
; Read
; -------------------------------------------------
VR_ReqRead:
	sbrs	status,7
	rjmp	VR_NormalReturn
	ldi	inttemp,READY_MSG
	sts	dispmark,inttemp
	ldi	YL,INTENBL_R
	ld	inttemp,Y
	cbr	inttemp,EP2_RDY_INTR	;; DISABLE EP2
	ldi	YL,INTENBL_W
	st	Y,inttemp
	st	X+,status		;; store status
	clr	status			;; status clear
	rjmp	VR_NormalReturn

; --------------------------------------------------------
; After EP0 return setting,goto  FuncReturn
; inttemp: xmitsize
; (ALREADY MUST be set data in tx-buffer
; --------------------------------------------------------
VR_ReqNoXmitReturn:
	clr	inttemp
VR_ReqXmitReturn:
	ori	status,0x40		;; xmitting.
	sts	EP0xmitsize,inttemp	
	ldi	inttemp,low(EP0xmitbuf)
	sts	EP0xmitptrL,inttemp
	ldi	inttemp,high(EP0xmitbuf)
	sts	EP0xmitptrH,inttemp
	ser	inttemp
	sts	EP0xmittype,inttemp	;; type is buf
	lds	inttemp,devwLengthL
;;	sts	dispchr,inttemp
	rjmp	Exec_FuncReturn

; --------------------------------------------------------
; USB Execute Status Stage Requests functions
; --------------------------------------------------------
ExecuteRequest:
	lds	inttemp,devbmRequestType
	andi	inttemp,0x60
	tst	inttemp
	breq	ExecReq_Standard

ExecReq_Class:
	cpi	inttemp,TYPE_OF_CLASS
	brne	ExecReq_Vendor

	;; Execute Stall
	ldi	YL,EP0STAT_W
	ldi	inttemp,0x04		;; EP0 STALL
	st	Y,inttemp		;; set
	rjmp	Exec_FuncReturn

ExecReq_Vendor:
	cpi	inttemp,TYPE_OF_VENDOR
	brne	Exec_FuncStall
	lds	inttemp,devbRequest
	rjmp	ExecuteVendorReq

; -------------------------------------------------------
; Standard USB Request 
; -------------------------------------------------------
ExecReq_Standard:		;; STANDARD TYPE?
	;; Standerd type
	lds	inttemp,devbRequest

;;	mov	r24,inttemp	;; for debug
;;	subi	r24,-$30

	cpi	inttemp,CLEAR_FEATURE
	brne	PC+2
	rjmp	Exec_ClearFeature
	cpi	inttemp,GET_DESCRIPTOR
	brne	PC+2
	rjmp	Exec_GetDescipter
	cpi	inttemp,GET_CONFIGRATION
	brne	PC+2
	rjmp	Exec_GetConfigration
	cpi	inttemp,GET_INTERFACE
	brne	PC+2
	rjmp	Exec_GetInterface
	cpi	inttemp,GET_STATUS
	brne	PC+2
	rjmp	Exec_GetStatus
	cpi	inttemp,SET_ADDRESS
	brne	PC+2
	rjmp	Exec_SetAddress
	cpi	inttemp,SET_CONFIGRATION
	brne	PC+2
	rjmp	Exec_SetConfigration
	cpi	inttemp,SET_DESCRIPTER
	brne	PC+2
	rjmp	Exec_SetDescripter
	cpi	inttemp,SET_FEATURE
	brne	PC+2
	rjmp	Exec_SetFeature
	cpi	inttemp,SET_INTERFACE
	brne	PC+2
	rjmp	Exec_SetInterface
	cpi	inttemp,SYNC_FRAME
	brne	PC+2
	rjmp	Exec_SyncFrame

; StaLL!!
Exec_FuncStall:
	;; Execute Stall
	ldi	YL,EP0STAT_W
	ldi	inttemp,0x04
	st	Y,inttemp		;; set

Exec_FuncReturn:

	ret

; ------------------------------------------------
; USB Remote Address setting.
; ------------------------------------------------
Exec_SetAddress:
	lds	inttemp,devwValueH
	tst	inttemp
	brne	Exec_Fail_Stall_1
	lds	inttemp,devwValueL
	cpi	inttemp,128
	brcc	Exec_Fail_Stall_1
	sts	usbaddress,inttemp	;; usbaddress=wValue
	rjmp	Exec_FuncReturn
Exec_Fail_Stall_1:
	rjmp	Exec_Fail_Stall

; ----------------------------------------------
; Clear Feature
; ----------------------------------------------
Exec_ClearFeature:
	lds	inttemp,devbmRequestType
	andi	inttemp,0x1f
	cpi	inttemp,DEVICE_RECIPIENT
	brne	Exec_ClrF_If
	lds	inttemp2,devwValueH	;; wValue upper 0x100
	brne	Exec_ClrF_If
	lds	inttemp2,devwValueL
	dec	inttemp2	;; if inttemp2=1 then 
	brne	Exec_ClrF_If
	clr	inttemp2
	sts	remotewakeup,inttemp2
	rjmp	Exec_FuncReturn
Exec_ClrF_If:
	cpi	inttemp,INTERFACE_RECICPENT
	brne	Exec_ClrF_Endp
	rjmp	Exec_Fail_Stall		;; if is defined.
Exec_ClrF_Endp:
	cpi	inttemp,ENDPOINT_RECICPENT
	brne	Exec_Fail_Stall_1
	;; endpoint_
	lds	inttemp,devwValueH
	lds	inttemp2,devwValueL
	or	inttemp,inttemp2
	brne	Exec_Fail_Stall_1	;; wValue<>0 then error.
	lds	inttemp,devwIndexL
	andi	inttemp,0x0f
	cpi	inttemp,3		;; Endpoint = ep0,1,2 ?
	brcc	Exec_Fail_Stall_1
	tst	inttemp
	brne	Exec_ClrF_Ep12
	;; EP0
	ldi	YL,EP0STAT_R
	ld	inttemp,Y
	ldi	YL,EP0STAT_W
	cbr	inttemp,0x04		;; Clear STALL
	st	Y,inttemp		;; set
	ldi	inttemp,0x01
	st	Y,inttemp		;; set
	rjmp	Exec_FuncReturn
Exec_ClrF_Ep12:
	dec	inttemp
	brne	Exec_ClrF_EP2
	;; EP1
	ldi	YL,EP1CON_R
	ld	inttemp,Y
	cbr	inttemp,0x02		;; Clear STALL
	ldi	YL,EP1CON_W
	st	Y,inttemp		;; set
	ldi	inttemp,0x01		;; DATA SEQUENCE
	ldi	YL,EP1TGL_W
	st	Y,inttemp
	rjmp	Exec_FuncReturn
Exec_ClrF_EP2:
	ldi	YL,EP2CON_R
	ld	inttemp,Y
	cbr	inttemp,0x02		;; Clear Stall
	ldi	YL,EP2CON_W
	st	Y,inttemp		;; set
	ldi	inttemp,0x01		;; DATA SEQUENCE BIT
	ldi	YL,EP2TGL_W
	st	Y,inttemp
	rjmp	Exec_FuncReturn

; ----------------------------------------------
; Get Current Configration
; ----------------------------------------------
Exec_GetConfigration:
	clr	inttemp2
	lds	inttemp,usbconfig
	cpi	inttemp,0xff
	breq	Exec_GetCfg_none
	mov	inttemp2,inttemp
Exec_GetCfg_none:
	sts	EP0xmitbuf,inttemp2
	ldi	inttemp,1
	sts	EP0xmitsize,inttemp	;; sendsize = 1
	sts	EP0xmittype,inttemp	;; type is buf
	ldi	inttemp,low(EP0xmitbuf)
	sts	EP0xmitptrL,inttemp
	ldi	inttemp,high(EP0xmitbuf)
	sts	EP0xmitptrH,inttemp
	rjmp	Exec_FuncReturn		;; go.

; ------------------------------------------------------
; Get Descriptor
; set FIFO write pointer
; ------------------------------------------------------
Exec_GetDescipter:
	lds	inttemp,devwValueH

	cpi	inttemp,DEV_DEVICE
	brne	Exec_GetDesc_Config
;
; Transmit Device Descriptor
;
	ldi	inttemp,18		;; descripter size
	sts	EP0xmitsize,inttemp
	clr	inttemp			;; PROGRAM ROM SELECT
	sts	EP0xmittype,inttemp
;;
	ldi	inttemp,low(2*DeviceDescriptor)
	sts	EP0xmitptrL,inttemp
	ldi	inttemp,high(2*DeviceDescriptor)
	sts	EP0xmitptrH,inttemp

	rjmp	Exec_FuncReturn
; ------------------------------------------
;; -----------------------------------------
Exec_GetDesc_Config:
;
; Transmit Configration Descriptor
;
	cpi	inttemp,DEV_CONFIGRATION
	brne	Exec_GetDesc_String

	lds	inttemp,devwValueL	;; Descriptor index
	tst	inttemp
	breq	Exec_GetDesc_Config_xmit
	;; STALL
	rjmp	Exec_Fail_Stall
Exec_GetDesc_Config_xmit:
;	ldi	inttemp,'g'		;; for debug.
;	sts	dispmark,inttemp

	ldi	inttemp,33		;; config descripter+if+endpoint size*2
	sts	EP0xmitsize,inttemp
	clr	inttemp			;; PROGRAM ROM SELECT
	sts	EP0xmittype,inttemp
;;
	ldi	inttemp,low(2*ConfigDescriptor)
	sts	EP0xmitptrL,inttemp
	ldi	inttemp,high(2*ConfigDescriptor)
	sts	EP0xmitptrH,inttemp
;;
	lds	inttemp,devwLengthH
	tst	inttemp			;; because of this program can xmit with max 255 bytes
	brne	Exec_FuncReturn_1
	lds	inttemp,devwLengthL
	mov	inttemp2,inttemp	;;
	lds	inttemp,EP0xmitsize
	cp	inttemp2,inttemp	;; reqsize >= xmitsize ?
	brcc	Exec_FuncReturn_1	;; ok.
	sts	EP0xmitsize,inttemp2	;; xmitsize := reqsize
Exec_FuncReturn_1:
	rjmp	Exec_FuncReturn

;
; Language ID or Manufacture/Product Strings
;
Exec_GetDesc_String:
Exec_Fail_Stall_2:
	rjmp	Exec_Fail_Stall

; -----------------------------------------------------------------
; Get Current Interface
; This program uses only EP1. So,always may check index is 0.
; -----------------------------------------------------------------
Exec_GetInterface:
	lds	inttemp,devwIndexH
	tst	inttemp
	brne	Exec_Fail_Stall_2
	lds	inttemp,devwIndexL
	tst	inttemp
	brne	Exec_Fail_Stall_2
	;; windex=0ȊO͑Sstall
	clr	inttemp
	sts	EP0xmitbuf,inttemp	;; set always 0
	inc	inttemp
	sts	EP0xmittype,inttemp
	sts	EP0xmitsize,inttemp	;; send 0x00
	ldi	inttemp,low(EP0xmitbuf)
	sts	EP0xmitptrL,inttemp
	ldi	inttemp,high(EP0xmitbuf)
	sts	EP0xmitptrH,inttemp
	rjmp	Exec_FuncReturn

; ---------------------------------------------------------
;  Get Status
; ---------------------------------------------------------
Exec_GetStatus:
	clr	inttemp
	sts	EP0xmitbuf,inttemp
	sts	EP0xmitbuf+1,inttemp
;;;
	lds	inttemp,devbmRequestType
	andi	inttemp,0x1f
	cpi	inttemp,0x00
	brne	Exec_GetStat_Interface
	lds	inttemp,remotewakeup
	sts	EP0xmitbuf,inttemp
	rjmp	Exec_GetStat_TxSet
Exec_GetStat_Interface:
	cpi	inttemp,INTERFACE_RECICPENT
	brne	Exec_GetStat_EndPoint
	rjmp	Exec_GetStat_TxSet
;;;
Exec_GetStat_EndPoint:
	cpi	inttemp,ENDPOINT_RECICPENT
	brne	Exec_GetStat_Stall
	lds	inttemp,devwIndexL
	andi	inttemp,0x0f
	cpi	inttemp,3
	brcc	Exec_GetStat_Stall	;; ENDPOINT Number = ep0,1,2
	tst	inttemp
	brne	Exec_GetStat_EP12
	;; ep0
	clr	inttemp2
	ldi	YL,EP0STAT_R
	ld	inttemp,Y
	sbrc	inttemp,2		;; STALL bit ?
	inc	inttemp2
	sts	EP0xmitbuf,inttemp
	rjmp	Exec_GetStat_TxSet
Exec_GetStat_EP12:
	cpi	inttemp,1
	brne	Exec_GetStat_EP2
	;; ep1
	ldi	YL,EP1CON_R
	ld	inttemp2,Y
	lds	inttemp,devwIndexL
	eor	inttemp,inttemp2
	andi	inttemp,DEVICEtoHOST
	brne	Exec_GetStat_Stall
	ldi	YL,EP1CON_R
	ld	inttemp,Y
	clr	inttemp2	;; stat = 0
	sbrc	inttemp,1
	inc	inttemp2	;; stat = 1
	sts	EP0xmitbuf,inttemp2
	rjmp	Exec_GetStat_TxSet
Exec_GetStat_EP2:
	;; ep2
	ldi	YL,EP2CON_R
	ld	inttemp2,Y
	lds	inttemp,devwIndexL
	eor	inttemp,inttemp2
	andi	inttemp,DEVICEtoHOST
	brne	Exec_GetStat_Stall
	ldi	YL,EP2CON_R
	ld	inttemp,Y
	clr	inttemp2	;; stat = 0
	sbrc	inttemp,1
	inc	inttemp2	;; stat = 1
	sts	EP0xmitbuf,inttemp2
	rjmp	Exec_GetStat_TxSet
Exec_GetStat_Stall:
	ldi	YL,EP0STAT_W
	ldi	inttemp,0x04
	st	Y,inttemp		;; Execute Stall.
Exec_GetStat_TxSet:
	ldi	inttemp,2
	sts	EP0xmitsize,inttemp	;; size=2
	sts	EP0xmittype,inttemp	;; type is buffer
	ldi	inttemp,low(EP0xmitbuf)
	sts	EP0xmitptrL,inttemp
	ldi	inttemp,high(EP0xmitbuf)
	sts	EP0xmitptrH,inttemp
	rjmp	Exec_FuncReturn

; ----------------------------------------------------
;  Set Configration
; ----------------------------------------------------
Exec_SetConfigration:
	lds	inttemp,devwValueH
	tst	inttemp
	brne	Exec_Fail_Stall_3
	lds	inttemp,devwValueL
	tst	inttemp
	brne	Exec_SetCf_chkindex
	;; wValue=0
	ldi	YL,EP1CON_R
	ld	inttemp,Y
	cbr	inttemp,DATA_SET
	ldi	YL,EP1CON_W
	st	Y,inttemp
	ldi	YL,EP2CON_R
	ld	inttemp,Y
	cbr	inttemp,DATA_SET
	ldi	YL,EP2CON_W
	st	Y,inttemp
	ser	inttemp
	sts	usbconfig,inttemp	;; set nosetting
	rjmp	Exec_FuncReturn
Exec_SetCf_chkindex:
	cpi	inttemp,1
	brne	Exec_Fail_Stall_3
	sts	usbconfig,inttemp	;; config is 1
	clr	inttemp
	sts	devwIndexL,inttemp
	sts	devwIndexH,inttemp
	sts	devwValueL,inttemp
	sts	devwValueH,inttemp
	rjmp	Exec_SetInterface	;; return at jumped routine
;;	rjmp	Exec_FuncReturn

; ---------------------------------------------------------
; Set Feature
; ---------------------------------------------------------
Exec_SetFeature:
;	ldi	inttemp,'f'		;; for debug.
;	sts	dispmark,inttemp

	lds	inttemp,devbmRequestType
	andi	inttemp,0x1f
	cpi	inttemp,DEVICE_RECIPIENT
	brne	Exec_SetF_If
	lds	inttemp2,devwValueH
	brne	Exec_SetF_If
	lds	inttemp2,devwValueL
	dec	inttemp2	;; if inttemp2=1 then 
	brne	Exec_SetF_If
	ldi	inttemp,0x01
	sts	remotewakeup,inttemp
	rjmp	Exec_FuncReturn
Exec_SetF_If:
	cpi	inttemp,INTERFACE_RECICPENT
	brne	Exec_SetF_Endp
Exec_Fail_Stall_3:
	rjmp	Exec_Fail_Stall		;; if is defined.
Exec_SetF_Endp:
	cpi	inttemp,ENDPOINT_RECICPENT
	brne	Exec_Fail_Stall_3
	;; endpoint_
	lds	inttemp,devwValueH
	lds	inttemp2,devwValueL
	or	inttemp,inttemp2
	brne	Exec_Fail_Stall_3	;; wValue<>0 then error.
	lds	inttemp,devwIndexL
	andi	inttemp,0x0f
	cpi	inttemp,3		;; Endpoint = ep0,1,2 ?
	brcc	Exec_Fail_Stall_3
	tst	inttemp
	brne	Exec_SetF_Ep12
	;; EP0
	ldi	YL,EP0STAT_R
	ld	inttemp,Y
	cbr	inttemp,0x04		;; Clear STALL
	ldi	YL,EP0STAT_W
	st	Y,inttemp		;; set
	rjmp	Exec_FuncReturn
Exec_SetF_Ep12:
	dec	inttemp
	brne	Exec_SetF_EP2
	;; EP1
	ldi	YL,EP1CON_R
	ld	inttemp,Y
	sbr	inttemp,0x02		;; Set STALL
	ldi	YL,EP1CON_W
	st	Y,inttemp		;; set
	rjmp	Exec_FuncReturn
Exec_SetF_EP2:
	ldi	YL,EP2CON_R
	ld	inttemp,Y
	sbr	inttemp,0x02		;; Set Stall
	ldi	YL,EP2CON_W
	st	Y,inttemp		;; 
;;
	
	rjmp	Exec_FuncReturn

; ------------------------------------------------------
; Set Interface 
; This program has only ONE Interface.
; So, it have only a interface with wIndex=0
; ------------------------------------------------------
Exec_SetInterface:
;	ldi	inttemp,'i'		;; for debug.
;	sts	dispmark,inttemp

	lds	inttemp,devwIndexH
	tst	inttemp
	brne	Exec_SetIf_Stall
;;
	lds	inttemp,devwIndexL
	tst	inttemp
	brne	Exec_SetIf_Stall
;; Setting Interface
;; ENDPOINT 1
	ldi	YL,EP1CON_R
	ld	inttemp,Y
	cbr	inttemp,DATA_SET  ;; RESET BIT
	ldi	YL,EP1CON_W
	st	Y,inttemp
;; ENDPOINT 2
	ldi	YL,EP2CON_R
	ld	inttemp,Y
	cbr	inttemp,DATA_SET  ;; RESET BIT
	ldi	YL,EP2CON_W
	st	Y,inttemp
;; set transmit/receive
	ldi	YL,EP1CON_W
	ldi	inttemp,0x00 | 0x01	;; HOST to DEVICE | CONFIG_BIT
	st	Y,inttemp
;; set payload size
	ldi	YL,EP1PLD_W
	ldi	inttemp,64	;; 64byte
	st	Y,inttemp
; init data-sequence bit
	ldi	YL,EP1TGL_W
	ldi	inttemp,DATA_SET ;; SET
	st	Y,inttemp
;; ENDPOINT 2
;; set transmit/receive
	ldi	YL,EP2CON_W
	ldi	inttemp,0x80 | 0x01	;; DEVICE to HOST | CONFIG_BIT
	st	Y,inttemp
;; set payload size
	ldi	YL,EP2PLD_W
	ldi	inttemp,64	;; 64byte
	st	Y,inttemp
; init data-sequence bit
	ldi	YL,EP2TGL_W
	ldi	inttemp,DATA_SET ;; SET
	st	Y,inttemp
; clear EP2-txfifo
	ldi	YL,CLRFIFO
	ldi	inttemp,0x04	;; EP2 Clear
	st	Y,inttemp
	ldi	inttemp,READY_MSG
	sts	dispmark,inttemp
	rjmp	Exec_FuncReturn
Exec_SetIf_Stall:

; ---------------------------------
; no support
; ---------------------------------
Exec_SyncFrame:
Exec_SetDescripter:
Exec_Fail_Stall:
	ldi	YL,EP0STAT_W
	ldi	inttemp,0x04
	st	Y,inttemp		;; set
	rjmp	Exec_FuncReturn

; --------------------------------------------------------
; USB Setup Stage
; --------------------------------------------------------
reqUSBSetup:

	ldi	XH,high(devbmRequestType)
	ldi	XL,low(devbmRequestType)
	ldi	YL,0xd0		;; bmRequest Register
;;
	ld	inttemp,Y+
	st	X+,inttemp	;; bmRequest
	ld	inttemp,Y+
	st	X+,inttemp	;; bRequest
	ld	inttemp,Y+
	st	X+,inttemp	;; wValue L
	ld	inttemp,Y+
	st	X+,inttemp	;; wValue H
	ld	inttemp,Y+
	st	X+,inttemp	;; wIndex L
	ld	inttemp,Y+
	st	X+,inttemp	;; wIndex H
	ld	inttemp,Y+
	st	X+,inttemp	;; wLength L
	ld	inttemp,Y+
	st	X+,inttemp	;; wLength H
;;
	ldi	YL,EP0STAT_W	
	ldi	inttemp,0x01	;; Set Reset
	st	Y,inttemp
;;
	clr	inttemp
	sts	EP0xmitsize,inttemp	;; Clear buf.
	sts	EP0recvsize,inttemp	;; 
	sts	EP0recved,inttemp	;;

	lds	inttemp,devwLengthH
	tst	inttemp
	brne	stat_CheckReqType
	lds	inttemp,devwLengthL
	tst	inttemp
	brne	stat_CheckReqType
	;; wLength==0

	;; Entry Status Stage ?
	ldi	YL,EP0STAT_R
stat_CheckStatusStage:
	ld	inttemp,Y
	andi	inttemp,0x18
	breq	stat_CheckStatusStage
;;
	rcall	ExecuteRequest
;;
	ldi	YL,PKTRDY_W
	ldi	inttemp,0x10
	st	Y,inttemp	;; PKTRDY EP0 TxRdy Set

	cbr	status,0x40	;; clear status

	ldi	YL,PKTRDY_R
stat_CheckTxAck:
	ld	inttemp,Y
	andi	inttemp,0x10
	brne	stat_CheckTxAck	;; ACK Received?
;;
	;; Set Address exception
	lds	inttemp,usbaddress
	tst	inttemp
	breq	reqReturn_
	mov	inttemp2,inttemp
	ldi	YL,DVCADR_R
	ld	inttemp,Y
	cp	inttemp,inttemp2
	breq	reqReturn_
	ldi	YL,DVCADR_W
	st	Y,inttemp2		;; set new address
	clr	inttemp2
	sts	usbaddress,inttemp2	;; usbaddress=0

;	ldi	inttemp,0x80	;; for debug
;	sts	dispmark,inttemp

reqReturn_:
	rjmp	reqReturn	;; end.

stat_CheckReqType:
	lds	inttemp,devbmRequestType
	andi	inttemp,DEVICEtoHOST
	breq	stat_HostToDevice
;; 
;; Transmit Request (Device to Host)
;; 
	rcall	ExecuteRequest
	;; Request Transmitter
	ldi	YL,INTENBL_R
	ld	inttemp,Y
	sbr	inttemp,EP0_XMIT_INTR
	ldi	YL,INTENBL_W
	st	Y,inttemp
	rjmp	reqReturn
;; 
;; Receive Request
;; 
stat_HostToDevice:
	ldi	YL,INTENBL_R
	ld	inttemp,Y
	sbr	inttemp,EP0_RECV_INTR
	ldi	YL,INTENBL_W
	st	Y,inttemp
	rjmp	reqReturn	;; end.

; ------------------------------------------------------------------
; EP0 ONLY
; This program is sent only under 16bytes only with EP0 IN Control
; ------------------------------------------------------------------
writeFIFO:
	ldi	YL,PKTRDY_R
	ld	inttemp,Y	;; read packet ready
	andi	inttemp,0x10
	brne	writeFIFO_exit
	lds	inttemp2,EP0xmitsize	;; inttemp2=transcnt
	tst	inttemp2
	breq	writeFIFO_stoptx

;	mov	inttemp,inttemp2	;; for debug
;	subi	inttemp,-$20
;	sts	dispmark,inttemp

	ldi	YL,EP0TXPLD_R
	ld	inttemp,Y
	cp	inttemp,inttemp2
	brcc	writeFIFO_nofix
	mov	inttemp2,inttemp
	;; inttemp2 is transmit size
writeFIFO_nofix:
	mov	inttemp3,inttemp2	;; inttemp3 is backuped xmit bytes
	lds	ZL,EP0xmitptrL
	lds	ZH,EP0xmitptrH
	ldi	YL,EP0TXFIFO
	lds	inttemp,EP0xmittype
	tst	inttemp
	brne	writeFIFO_frombuf
	;; from PROGRAM ROM
writeFIFO_writeloop1:
	lpm
	st	Y,r0
	adiw	ZL,1
	dec	inttemp2
	brne	writeFIFO_writeloop1
	rjmp	writeFIFO_resetbytes
writeFIFO_frombuf:
	;; using LOCAL Buffer
	ld	inttemp,Z+
	st	Y,inttemp
	dec	inttemp2
	brne	writeFIFO_frombuf
writeFIFO_resetbytes:
	sts	EP0xmitptrL,ZL
	sts	EP0xmitptrH,ZH
	lds	inttemp2,EP0xmitsize
	sub	inttemp2,inttemp3
	sts	EP0xmitsize,inttemp2
	ldi	YL,PKTRDY_W
	ldi	inttemp,0x10
	st	Y,inttemp	;; PKTRDY <- EP0 TX RDY SET
	rjmp	writeFIFO_exit
; disable EP0 tx interrupt
writeFIFO_stoptx:
	ldi	YL,INTENBL_R
	ld	inttemp,Y
	cbr	inttemp,EP0_XMIT_INTR
	ldi	YL,INTENBL_W	;; Disable EP0 TX INTR
	st	Y,inttemp
	ldi	YL,PKTRDY_W
	ldi	inttemp,0x01	;; PKTRDT <- EP0 RX RDY RESET
	st	Y,inttemp
	cbr	status,0x40	;; clear status tx flag.

writeFIFO_exit:

	ret

; ----------------------------------------------------------
; write FIFO to Endpoint 2
; This routine is used only for reading Flash memory
; ----------------------------------------------------------
writeFIFO_EP2:
	ldi	YL,PKTRDY_R
	ld	inttemp,Y	;; read packet ready
	andi	inttemp,0x40
	brne	writeFIFOEP2_exit

	mov	inttemp,status
	andi	inttemp,0x3f
	cpi	inttemp,_STATUS_OF_READ
	brne	writeFIFOEP2_stoptx
	sbrc	status,6
	ret			;; check ep0 status return execution.

	ldi	YL,EP2PLD_R
	ld	inttemp2,Y	;; transfer size

	ldi	YL,EP2TXFIFO

	sbrc	status,7
	rjmp	trans_nullsend	;;

	ldi	ZH,0x02
	mov	ZL,adr0
trans_countloop:
	ld	inttemp,Z+
	st	Y,inttemp
	inc	inttemp3
	tst	ZL
	brne	trans_countup

	add	adr1,one
	adc	adr2,zero
	ldi	ZH,0x04
	mov	ZL,adr1		; address load
	ld	inttemp,Z
	ldi	ZH,0x08
	mov	ZL,adr2
	ld	inttemp,Z	; address load
	clr	ZL
	ldi	ZH,0x02		; low set
	cp	adr2,adrstop
	breq	writeFIFOEP2_stoptx
trans_countup:
	dec	inttemp2
	brne	trans_countloop

	mov	adr0,ZL		; low

	ldi	YL,PKTRDY_W
	ldi	inttemp,0x40
	st	Y,inttemp	;; PKTRDY <- EP2 TX RDY SET

	ret			;; 

;-------------------------------------------------
; disable EP2 tx interrupt
;-------------------------------------------------
writeFIFOEP2_stoptx:		;; last check
	dec	inttemp2
	brne	trans_nullsend
;; 
	ldi	YL,PKTRDY_W
	ldi	inttemp,0x40
	st	Y,inttemp	;; PKTRDY <- EP2 TX RDY SET
	rjmp	writeFIFOEP2_disable

trans_nullsend:
	tst	inttemp2
	breq	writeFIFOEP2_disable
trans_dummyloop:
	st	Y,zero
	dec	inttemp2
	brne	trans_dummyloop

	ldi	YL,PKTRDY_W
	ldi	inttemp,0x40
	st	Y,inttemp	;; PKTRDY <- EP2 TX RDY SET

;	ldi	YL,INTENBL_R
;	ld	inttemp,Y
;	cbr	inttemp,EP2_RDY_INTR
;	ldi	YL,INTENBL_W	;; Disable EP2 TX INTR
;	st	Y,inttemp
writeFIFOEP2_disable:
	sbr	status,0x80		;; fin.

writeFIFOEP2_exit:

	ret

; -----------------------------------------------------------
; Read FIFO from EndPoint 0
; -----------------------------------------------------------
readFIFO_EP0:
	lds	inttemp,EP0recved
	tst	inttemp		;; if not finish receiving
	brne	readFIFOEP0_exit
	lds	inttemp,EP0recvsize
	ldi	YL,EP0RXCNT
	ld	inttemp2,Y	;; inttemp2 = recved size
	add	inttemp,inttemp2
	brcs	readFIFOEP0_exit	;; OverFlow
	cpi	inttemp,33
	brcc	readFIFOEP0_exit	;; Carry
	lds	inttemp2,EP0recvsize	;; recved size
	sts	EP0recvsize,inttemp	;; New Recved size
	ldi	ZL,low(EP0recvbuf)
	ldi	ZH,high(EP0recvbuf)
	add	ZL,inttemp2
	clr	inttemp2
	adc	ZH,inttemp2
	ld	inttemp2,Y	;; Load Counter
	tst	inttemp2
	breq	readFIFO_notrecv
	ldi	YL,EP0RXFIFO
readFIFO_loop1:
	ld	inttemp,Y
	st	Z+,inttemp	;; copy buf.
	dec	inttemp2
	brne	readFIFO_loop1
readFIFO_notrecv:
	ldi	YL,PKTRDY_W
	ldi	inttemp,0x01
	st	Y,inttemp	;; PKTRDY <- SET EP0 RX RDY
	lds	inttemp2,devwLengthL
	tst	inttemp2	;; wLengthL=0?
	breq	readFIFOEP0_clrexit
	;; wLength<>0
	lds	inttemp,devwLengthH
	tst	inttemp		;; wLengthH=0?
	brne	readFIFOEP0_clrexit	;; Over;)
	lds	inttemp,EP0recvsize
	cp	inttemp,inttemp2	;; wLength=recvedsize?
	brne	readFIFOEP0_clrexit
;;
;;      ok.
	ser	inttemp
	sts	EP0recved,inttemp	;; received flag.
	rcall	ExecuteRequest
	ldi	YL,PKTRDY_W
	ldi	inttemp,0x10
	st	Y,inttemp		;; PKTRDY <- SET EP0 TX RDY
	ldi	YL,INTENBL_R
	ld	inttemp,Y
	cbr	inttemp,EP0_RECV_INTR	;; DISABLE EP0 RX ENABLE
	ldi	YL,INTENBL_W
	st	Y,inttemp
readFIFOEP0_clrexit:
	cbr	status,0x40

readFIFOEP0_exit:

	ret

; -------------------------------------------------------
; Read FIFO from EndPoint 1
; This Endpoint used only transfering from PC to Flash.
; -------------------------------------------------------
readFIFO_EP1:
	ldi	YL,EP1RXCNT
	ld	inttemp,Y	;; inttemp2 = recved size
	cpi	inttemp,65
	brcs	PC+2
	ldi	inttemp,64	;; max 64.(perhaps not happened:)
	ldi	ZL,low(EP1recvbuf)
	ldi	ZH,high(EP1recvbuf)
	tst	inttemp
	breq	readFIFO_notrecv2	;; recved size = 0?
	ldi	YL,EP1RXFIFO
	mov	inttemp3,inttemp	;; backup.
readFIFO_loop2:
	ld	inttemp2,Y
	st	Z+,inttemp2	;; copy buf.
	dec	inttemp
	brne	readFIFO_loop2
;;
readFIFO_notrecv2:
	ldi	YL,PKTRDY_W	;;
	ldi	inttemp,0x02
	st	Y,inttemp	;; PKTRDY <- SET EP1 RX RDY
;;
; if status is not _STATUS_OF_WRITE, i don't execute write data to flash.
	mov	inttemp,status
	andi	inttemp,0x3f
	cpi	inttemp,_STATUS_OF_WRITE
	brne	ignorewrite	;; not mode select.
	tst	inttemp3	;; writesize = 0 ?
	brne	PC+2
ignorewrite:
	ret
;
; write execution
; inttemp3 is recvsize.
	sbrc	status,7
	rjmp	flashwriteerr_exit	;; if already fin,request is ignore

	ldi	XL,low(EP1recvbuf)
	ldi	XH,high(EP1recvbuf)

;----------------------------------------------------
flashwriteloop:
	tst	accmode
	brne	flashwriteloop_B
	
;----------------------------------------------------
; Write Block A
;----------------------------------------------------
flashwriteloop_A:
	ld	inttemp2,X+
	cpi	inttemp2,0xff		; if data is 0xFF then skip writing.
	breq	flashwrite_skipA
	rcall	Flash_WriteData_A
	rcall	Flash_WaitToggle
	brcs	flashwriteerr_exit
flashwrite_skipA:
	inc	adr0
	breq	flash_r_updatehighadr
	dec	inttemp3
	brne	flashwriteloop_A
	ret

;----------------------------------------------------
; Write Block B
;----------------------------------------------------
flashwriteloop_B:
	ld	inttemp2,X+
	cpi	inttemp2,0xff
	breq	flashwrite_skipB
	rcall	Flash_WriteData_B
	rcall	Flash_WaitToggle
	brcs	flashwriteerr_exit
flashwrite_skipB:
	inc	adr0
	breq	flash_r_updatehighadr
	dec	inttemp3
	brne	flashwriteloop_B
	ret

;----------------------------------------------------
;; update a8-a23 address latch.
;----------------------------------------------------
flash_r_updatehighadr:
	add	adr1,one
	adc	adr2,zero
	ldi	ZH,0x08	
	mov	ZL,adr2
	ld	inttemp,Z	; address load(top address only)
	clr	adr0
	cp	adr2,adrstop
	breq	writeflashstop	; function end.
	dec	inttemp3
	brne	flashwriteloop
	ret			; fin.

;----------------------------------------------------
; EP1 ready interrupt disable.
;----------------------------------------------------
flashwriteerr_exit:
	ldi	ZH,0x06
	ldi	ZL,0xAA
	ldi	inttemp,0xF0	; reset.
	st	Z,inttemp
	st	Z,inttemp
writeflashstop:
	sbr	status,0x80

	ret

; ----------------------------------------------------------------
; Send Chip Erase Command(A)
; ----------------------------------------------------------------
Flash_ChipErase_A:
	ldi	ZH,0x06
; Reset.
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
; chip erase cmd
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x80	;; Data = 80
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x10	;; Data = 10
	st	Z,inttemp

	ret

; ----------------------------------------------------------------
; Send Chip Erase Command(B)
; ----------------------------------------------------------------
Flash_ChipErase_B:
	ldi	ZH,0x06
; Reset.
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
; chip erase cmd
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x80	;; Data = 80
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x10	;; Data = 10
	st	Z,inttemp

	ret

; ----------------------------------------------------------------
; Send Sector Erase Command(A)
; inttemp2  : SA8-15
; inttemp3  : SA16-23
; Hi Latch is broken.
; ----------------------------------------------------------------
Flash_SectorErase_A:
	ldi	ZH,0x06
; Reset
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
; sector erase cmd
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x80	;; Data = 80
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
;;; set sector address
	ldi	ZH,0x04		;; bit15-8
	mov	ZL,inttemp2	;; 
	ld	inttemp,Z	;; set
	ldi	ZH,0x08		;; bit23-16
	mov	ZL,inttemp3	;; 
	ld	inttemp,Z	;; set
;;
	clr	ZL
	ldi	ZH,0x02
	ldi	inttemp,0x30	;; Data = 30
	st	Z,inttemp
;;
	ret

; ----------------------------------------------------------------
; Send Sector Erase Command(B)
; inttemp2  : SA8-15
; inttemp3  : SA16-23
; Hi Latch is broken.
; ----------------------------------------------------------------
Flash_SectorErase_B:
	ldi	ZH,0x06
; Reset
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
; sector erase cmd
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x80	;; Data = 80
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
;;; set sector address
	ldi	ZH,0x04		;; bit15-8
	mov	ZL,inttemp2	;; 
	ld	inttemp,Z	;; set
	ldi	ZH,0x08		;; bit23-16
	mov	ZL,inttemp3	;; 
	ld	inttemp,Z	;; set
;;
	clr	ZL
	ldi	ZH,0x02
	ldi	inttemp,0x30	;; Data = 30
	st	Z,inttemp
;;
	ret

; ----------------------------------------------------------------
; Send Data Write Command(A)
; inttemp2 : writing data.
; ----------------------------------------------------------------
Flash_WriteData_A:
; Reset
	ldi	ZH,0x06
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
; sector erase cmd
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xA0	;; Data = A0
	st	Z,inttemp
; MUST be SET Hi address(A23-A16) 
	ldi	ZH,0x04
	mov	ZL,adr1
	ld	inttemp,Z	; set address latch 1
	ldi	ZH,0x02
	mov	ZL,adr0
	st	Z,inttemp2	;; Addr <- data
;;
	ret

; ----------------------------------------------------------------
; Send Data Write Command(B)
; inttemp2 : writing data.
; ----------------------------------------------------------------
Flash_WriteData_B:
; Reset
	ldi	ZH,0x06
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0xF0	;; Data = F0
	st	Z,inttemp
; sector erase cmd
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0xAA	;; Data = AA
	st	Z,inttemp
	ldi	ZL,0xAA		;; Address = xxAAAA
	ldi	inttemp,0x55	;; Data = 55
	st	Z,inttemp
	ldi	ZL,0x55		;; Address = xx5555
	ldi	inttemp,0xA0	;; Data = A0
	st	Z,inttemp
; MUST be SET Hi address(A23-A16) 
	ldi	ZH,0x04
	mov	ZL,adr1
	ld	inttemp,Z	; set address latch 1
	ldi	ZH,0x02
	mov	ZL,adr0
	st	Z,inttemp2	;; Addr <- data
;;
	ret
;;
; ----------------------------------------------------------------
; Flash Toggle bit wait
; ----------------------------------------------------------------
Flash_WaitToggle:
	ldi	ZH,0x02
	mov	ZL,adr0
;;
WaitToggle_Loop:
	ld	inttemp,Z	;; 1st data
	ld	inttemp2,Z	;; 2nd data
	eor	inttemp2,inttemp
	sbrs	inttemp2,6	;; Bit6 is Zero?
	rjmp	WaitToggle_ok	;; Toggle cleared?
	sbrs	inttemp,5	;; 1st Data DQ5 <> 0?(not timeover)
	rjmp	WaitToggle_Loop
;;
	ld	inttemp,Z	;; 1st data
	ld	inttemp2,Z	;; 2nd data
	eor	inttemp2,inttemp
	sbrc	inttemp2,6
	rjmp	WaitToggle_ng	;; Toggle cleared?
WaitToggle_ok:
	clc	; no carry
	ret
WaitToggle_ng:
	sec	; with carry
	ret

;;
; ----------------------------------------------------------------
; Check Flash Toggle bit(for chiperase)
; with carry -> ERROR(NG)
; with    zf -> fin
; without zf -> not fin
; ----------------------------------------------------------------
Flash_CheckToggle:
	ldi	ZH,0x02
	mov	ZL,adr0
;;
	ld	inttemp,Z	;; 1st data
	ld	inttemp2,Z	;; 2nd data
	eor	inttemp2,inttemp
	sbrs	inttemp2,6	;; Bit6 is Zero?
	rjmp	CheckToggle_ok	;; Toggle cleared?
	sbrs	inttemp,5	;; 1st Data DQ5 <> 0?(not timeover)
	rjmp	CheckToggle_notend
;;
	ld	inttemp,Z	;; 1st data
	ld	inttemp2,Z	;; 2nd data
	eor	inttemp2,inttemp
	sbrc	inttemp2,6
	rjmp	CheckToggle_ng	;; Toggle cleared?
CheckToggle_ok:
	clc	; no carry
	sez	; zero
	ret
CheckToggle_notend:
	clc	; no carry
	clz	; nozero
	ret
CheckToggle_ng:
	sec	; with carry
	clz	; nozero
	ret

; ----------------------------------------------------------------
; Execute EP0 xmit Interrupt
; ----------------------------------------------------------------
reqEP0execXmit:
	rcall	writeFIFO
	rjmp	reqReturn

; ----------------------------------------------------------------
; Execute EP0 recv Interrupt
; ----------------------------------------------------------------
reqEP0execRecv:
	rcall	readFIFO_EP0
	rjmp	reqReturn

; ----------------------------------------------------------------
; Execute EP1 Interrupt
; ----------------------------------------------------------------
reqEP1execute:
	rcall	readFIFO_EP1
	rjmp	reqReturn

; ----------------------------------------------------------------
; Execute EP2 Interrupt
; ----------------------------------------------------------------
reqEP2Execute:
;	ldi	inttemp,'T'
;	sts	dispmark,inttemp
	rcall	writeFIFO_EP2
	rjmp	reqReturn

;; -------------------------------------------------------
;; interrupt vector 0
;; -------------------------------------------------------
INT0_VECT:
	in	stack,SREG

	push	ZH
	push	ZL
	ldi	YL,INTSTAT	;; Read INTSTAT
	ld	inttemp,Y

	;; INTERRUPT REQUSET
	sbrc	inttemp,0	;; USB Setup Request ?
	rjmp	reqUSBSetup
	sbrc	inttemp,1	;; USB EP1 ready
	rjmp	reqEP1execute
	sbrc	inttemp,2
	rjmp	reqEP2execute
	sbrc	inttemp,3
	rjmp	reqEP0execRecv
	sbrc	inttemp,4
	rjmp	reqEP0execXmit
	sbrc	inttemp,5	;; USB Bus reset?
	rjmp	reqUSBBusReset
;;	sbrc	inttemp,6
;;	rjmp	reqSuspend
;;	sbrc	inttemp,7
;;	rjmp	reqEP3


reqReturn:
	pop	ZL
	pop	ZH
	out	SREG,stack
	
	reti

initmsg:
	.db	'U','S','B',' ','W','A','I','T','I','N','G','.',0

readymsg:
	.db	'R','E','A','D','Y','.',0

chiperasemsg:
	.db	'C','H','I','P',' ','E','R','A','S','I','N','G','.',0

blankchecking:
	.db	'B','L','A','N','K',' ','C','H','E','C','K','I','N','G','.',0

writingmsg:
	.db	'W','R','I','T','I','N','G','.','.','.',0

chksummsg:
	.db	'C','A','L','C',' ','C','H','E','C','K','S','U','M','.',0

readingmsg:
	.db	'R','E','A','D','I','N','G','.','.','.',0

DeviceDescriptor:
	.db	18, DEV_DEVICE, 0, 1, 0, 0, 0, 8 , 0xD7,0x04
	.db	0x00,0x20,0x00, 0x01, 0, 0, 0, 1

;; AVR Assembler allocates Program Data with only EVEN allignments
ConfigDescriptor:
;; interfaces=1,index=1,string=1,BusPower,150mA ;;; if-desc = 9
	.db	9,DEV_CONFIGRATION, 32, 0, 1, 1, 0, 0x80, 75 , 9

InterfaceDescripter:
;; protocol is vender unique,endpoints=2,
	.db	DEV_INTERFACE,0,0,2,0xff,0xff,0xff,0

EndpointDescripter:
;; EP1 OUT BULK 64bytes
	.db	7, DEV_ENDPOINT, 0x01, 0x02, 64, 0, 0, 7
	.db	DEV_ENDPOINT, 0x82, 0x02, 64, 0, 0,0,0

.DSEG
; internal SRAM area

	.org	0x60

hexl:	.BYTE	1
hexh:	.BYTE	1

dispchr:	.BYTE	1
prevchr:	.BYTE	1
dispmark:	.BYTE	1
prevmark:	.BYTE	1

; Device Descriptor
devbmRequestType:	.BYTE	1
devbRequest:		.BYTE	1
devwValueL:		.BYTE	1
devwValueH:		.BYTE	1
devwIndexL:		.BYTE	1
devwIndexH:		.BYTE	1
devwLengthL:		.BYTE	1
devwLengthH:		.BYTE	1

usbaddress:	.BYTE	1
usbconfig:	.BYTE	1
remotewakeup:	.BYTE	1
;;
;; USB Endpoint Receive Buffers
;; xmittype or recvtype = 0 : PROGRAM ROM, else SRAM buffer
;; 
EP0xmitsize:	.BYTE 1
EP0xmittype:	.BYTE 1
EP0xmitptrL:	.BYTE 1
EP0xmitptrH:	.BYTE 1
EP0recvsize:	.BYTE 1
EP0recved:	.BYTE 1

EP2xmitsize:	.BYTE 1
EP2xmitptrL:	.BYTE 1
EP2xmitptrH:	.BYTE 1

;; EP2 is not use(ROM Data Transfer Only)
EP0recvbuf:	.BYTE	32
EP0xmitbuf:	.BYTE	16
EP1recvbuf:	.BYTE	64

vendorstr:	.BYTE	17
