unit USB100;
//
//   USB rev.1.0 Defination (from Win98DDK usb100.h)
//    programmed by M.yanagisawa.
//   }N͊֐WJĂ܂B
//
interface

uses
  Windows;

type
  PVOID = Pointer;
  USHORT = WORD;

const
  MAXIMUM_USB_STRING_LENGTH = 255;

/// values for the bits returned by the USB GET_STATUS command
  USB_GETSTATUS_SELF_POWERED = $01;
  USB_GETSTATUS_REMOTE_WAKEUP_ENABLED = $02;

  USB_DEVICE_DESCRIPTOR_TYPE = $01;
  USB_CONFIGURATION_DESCRIPTOR_TYPE = $02;
  USB_STRING_DESCRIPTOR_TYPE = $03;
  USB_INTERFACE_DESCRIPTOR_TYPE = $04;
  USB_ENDPOINT_DESCRIPTOR_TYPE = $05;
  USB_POWER_DESCRIPTOR_TYPE = $06;

function USB_DESCRIPTOR_MAKE_TYPE_AND_INDEX(d:WORD;i:WORD):WORD;

//
/// Values for bmAttributes field of an
/// endpoint descriptor
//

const
  USB_ENDPOINT_TYPE_MASK = $03;

  USB_ENDPOINT_TYPE_CONTROL = $00;
  USB_ENDPOINT_TYPE_ISOCHRONOUS = $01;
  USB_ENDPOINT_TYPE_BULK = $02;
  USB_ENDPOINT_TYPE_INTERRUPT = $03;


//
/// definitions for bits in the bmAttributes field of a
/// configuration descriptor.
//
const
  USB_CONFIG_POWERED_MASK = $c0;

  USB_CONFIG_BUS_POWERED = $80;
  USB_CONFIG_SELF_POWERED = $40;
  USB_CONFIG_REMOTE_WAKEUP = $20;

//
/// Endpoint direction bit, stored in address
//

const
  USB_ENDPOINT_DIRECTION_MASK = $80;

/// test direction bit in the bEndpointAddress field of
/// an endpoint descriptor.
function USB_ENDPOINT_DIRECTION_OUT(addr:UCHAR):boolean;
function USB_ENDPOINT_DIRECTION_IN(addr:UCHAR):boolean;

//
/// USB defined request codes
/// see chapter 9 of the USB 1.0 specifcation for
/// more information.
//

/// These are the correct values based on the USB 1.0
/// specification

const
  USB_REQUEST_GET_STATUS = $00;
  USB_REQUEST_CLEAR_FEATURE = $01;

  USB_REQUEST_SET_FEATURE = $03;

  USB_REQUEST_SET_ADDRESS = $05;
  USB_REQUEST_GET_DESCRIPTOR = $06;
  USB_REQUEST_SET_DESCRIPTOR = $07;
  USB_REQUEST_GET_CONFIGURATION = $08;
  USB_REQUEST_SET_CONFIGURATION = $09;
  USB_REQUEST_GET_INTERFACE = $0A;
  USB_REQUEST_SET_INTERFACE = $0B;
  USB_REQUEST_SYNC_FRAME = $0C;


//
/// defined USB device classes
//


const
  USB_DEVICE_CLASS_RESERVED = $00;
  USB_DEVICE_CLASS_AUDIO = $01;
  USB_DEVICE_CLASS_COMMUNICATIONS = $02;
  USB_DEVICE_CLASS_HUMAN_INTERFACE = $03;
  USB_DEVICE_CLASS_MONITOR = $04;
  USB_DEVICE_CLASS_PHYSICAL_INTERFACE = $05;
  USB_DEVICE_CLASS_POWER = $06;
  USB_DEVICE_CLASS_PRINTER = $07;
  USB_DEVICE_CLASS_STORAGE = $08;
  USB_DEVICE_CLASS_HUB = $09;
  USB_DEVICE_CLASS_VENDOR_SPECIFIC = $FF;

//
/// USB defined Feature selectors
//

const
  USB_FEATURE_ENDPOINT_STALL = $0000;
  USB_FEATURE_REMOTE_WAKEUP = $0001;
  USB_FEATURE_POWER_D0 = $0002;
  USB_FEATURE_POWER_D1 = $0003;
  USB_FEATURE_POWER_D2 = $0004;
  USB_FEATURE_POWER_D3 = $0005;

type
  _USB_DEVICE_DESCRIPTOR = packed record
    bLength: UCHAR;
    bDescriptorType: UCHAR;
    bcdUSB: USHORT;
    bDeviceClass: UCHAR;
    bDeviceSubClass: UCHAR;
    bDeviceProtocol: UCHAR;
    bMaxPacketSize0: UCHAR;
    idVendor: USHORT;
    idProduct: USHORT;
    bcdDevice: USHORT;
    iManufacturer: UCHAR;
    iProduct: UCHAR;
    iSerialNumber: UCHAR;
    bNumConfigurations: UCHAR;
  end {_USB_DEVICE_DESCRIPTOR};
  TUSB_DEVICE_DESCRIPTOR = _USB_DEVICE_DESCRIPTOR;
  PUSB_DEVICE_DESCRIPTOR = ^_USB_DEVICE_DESCRIPTOR;

type
  _USB_ENDPOINT_DESCRIPTOR = packed record
    bLength: UCHAR;
    bDescriptorType: UCHAR;
    bEndpointAddress: UCHAR;
    bmAttributes: UCHAR;
    wMaxPacketSize: USHORT;
    bInterval: UCHAR;
  end {_USB_ENDPOINT_DESCRIPTOR};
  TUSB_ENDPOINT_DESCRIPTOR = _USB_ENDPOINT_DESCRIPTOR;
  PUSB_ENDPOINT_DESCRIPTOR = ^_USB_ENDPOINT_DESCRIPTOR;

type
  _USB_CONFIGURATION_DESCRIPTOR = packed record
    bLength: UCHAR;
    bDescriptorType: UCHAR;
    wTotalLength: USHORT;
    bNumInterfaces: UCHAR;
    bConfigurationValue: UCHAR;
    iConfiguration: UCHAR;
    bmAttributes: UCHAR;
    MaxPower: UCHAR;
  end {_USB_CONFIGURATION_DESCRIPTOR};
  TUSB_CONFIGURATION_DESCRIPTOR = _USB_CONFIGURATION_DESCRIPTOR;
  PUSB_CONFIGURATION_DESCRIPTOR = ^_USB_CONFIGURATION_DESCRIPTOR;

type
  _USB_INTERFACE_DESCRIPTOR = packed record
    bLength: UCHAR;
    bDescriptorType: UCHAR;
    bInterfaceNumber: UCHAR;
    bAlternateSetting: UCHAR;
    bNumEndpoints: UCHAR;
    bInterfaceClass: UCHAR;
    bInterfaceSubClass: UCHAR;
    bInterfaceProtocol: UCHAR;
    iInterface: UCHAR;
  end {_USB_INTERFACE_DESCRIPTOR};
  TUSB_INTERFACE_DESCRIPTOR = _USB_INTERFACE_DESCRIPTOR;
  PUSB_INTERFACE_DESCRIPTOR = ^_USB_INTERFACE_DESCRIPTOR;

type
  _USB_STRING_DESCRIPTOR = packed record
    bLength: UCHAR;
    bDescriptorType: UCHAR;
    bString: Array[0..1-1] of WCHAR;
  end {_USB_STRING_DESCRIPTOR};
  TUSB_STRING_DESCRIPTOR = _USB_STRING_DESCRIPTOR;
  PUSB_STRING_DESCRIPTOR = ^_USB_STRING_DESCRIPTOR;

//
/// USB power descriptor added to core specification
//

const
  USB_SUPPORT_D0_COMMAND = $01;
  USB_SUPPORT_D1_COMMAND = $02;
  USB_SUPPORT_D2_COMMAND = $04;
  USB_SUPPORT_D3_COMMAND = $08;

  USB_SUPPORT_D1_WAKEUP = $10;
  USB_SUPPORT_D2_WAKEUP = $20;


type
  _USB_POWER_DESCRIPTOR = packed record
    bLength: UCHAR;
    bDescriptorType: UCHAR;
    bCapabilitiesFlags: UCHAR;
    EventNotification: USHORT;
    D1LatencyTime: USHORT;
    D2LatencyTime: USHORT;
    D3LatencyTime: USHORT;
    PowerUnit: UCHAR;
    D0PowerConsumption: USHORT;
    D1PowerConsumption: USHORT;
    D2PowerConsumption: USHORT;
  end {_USB_POWER_DESCRIPTOR};
  TUSB_POWER_DESCRIPTOR = _USB_POWER_DESCRIPTOR;
  PUSB_POWER_DESCRIPTOR = ^_USB_POWER_DESCRIPTOR;


type
  _USB_COMMON_DESCRIPTOR = packed record
    bLength: UCHAR;
    bDescriptorType: UCHAR;
  end {_USB_COMMON_DESCRIPTOR};
  TUSB_COMMON_DESCRIPTOR = _USB_COMMON_DESCRIPTOR;
  PUSB_COMMON_DESCRIPTOR = ^_USB_COMMON_DESCRIPTOR;


//
/// Standard USB HUB definitions
//
/// See Chapter 11
//

type
  _USB_HUB_DESCRIPTOR = packed record
    bDescriptorLength: UCHAR;
    bDescriptorType: UCHAR;
    bNumberOfPorts: UCHAR;
    wHubCharacteristics: USHORT;
    bPowerOnToPowerGood: UCHAR;
    bHubControlCurrent: UCHAR;
//
/// room for 255 ports power control and removable bitmask
    bRemoveAndPowerMask: Array[0..64-1] of UCHAR;
  end {_USB_HUB_DESCRIPTOR};
  TUSB_HUB_DESCRIPTOR = _USB_HUB_DESCRIPTOR;
  PUSB_HUB_DESCRIPTOR = ^_USB_HUB_DESCRIPTOR;

implementation

function USB_ENDPOINT_DIRECTION_OUT(addr:UCHAR):boolean;
begin
  Result:= (addr and USB_ENDPOINT_DIRECTION_MASK)=0;
end;

function USB_ENDPOINT_DIRECTION_IN(addr:UCHAR):boolean;
begin
  Result:= ((addr) and USB_ENDPOINT_DIRECTION_MASK)<>0;
end;

function USB_DESCRIPTOR_MAKE_TYPE_AND_INDEX(d:WORD;i:WORD):WORD;
begin
  Result:= (d SHL 8) or i;
end;

end.
