unit USBDI;

interface
uses
  Windows,usbioctl,usb100;

// anonymous structures have been removed, if your driver relies on these then }
// you must build with the following macro defined. }

{$DEFINE OSR21_COMPAT}

type
  PIRP = PVOID;
  PMDL = PVOID;

const
  USBDI_VERSION = $101;

//
// USB defined structures and constants 
// (see chapter 9 of USB specification)
//

const
  USB_DEFAULT_DEVICE_ADDRESS = 0;
  USB_DEFAULT_ENDPOINT_ADDRESS = 0;

//
// max packet size (bytes) for default endpoint
// until SET_ADDRESS command is received.
//

const
  USB_DEFAULT_MAX_PACKET = 64;

//
// USBD interface structures and constants
//


// Kernelmode Driver Only@myan.
//const
//  URB_FROM_IRP(Irp) = ((IoGetCurrentIrpStackLocation(Irp))->Parameters.Others.Argument1);

//
// URB request codes
//

const
  URB_FUNCTION_SELECT_CONFIGURATION = $0000;
  URB_FUNCTION_SELECT_INTERFACE = $0001;
  URB_FUNCTION_ABORT_PIPE = $0002;
  URB_FUNCTION_TAKE_FRAME_LENGTH_CONTROL = $0003;
  URB_FUNCTION_RELEASE_FRAME_LENGTH_CONTROL = $0004;
  URB_FUNCTION_GET_FRAME_LENGTH = $0005;
  URB_FUNCTION_SET_FRAME_LENGTH = $0006;
  URB_FUNCTION_GET_CURRENT_FRAME_NUMBER = $0007;
  URB_FUNCTION_CONTROL_TRANSFER = $0008;
  URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER = $0009;
  URB_FUNCTION_ISOCH_TRANSFER = $000A;
  URB_FUNCTION_RESET_PIPE = $001E;

//
// These functions correspond
// to the standard commands on the default pipe
//
// direction is implied
//

const
  URB_FUNCTION_GET_DESCRIPTOR_FROM_DEVICE = $000B;
  URB_FUNCTION_GET_DESCRIPTOR_FROM_ENDPOINT = $0024;
  URB_FUNCTION_GET_DESCRIPTOR_FROM_INTERFACE = $0028;

  URB_FUNCTION_SET_DESCRIPTOR_TO_DEVICE = $000C;
  URB_FUNCTION_SET_DESCRIPTOR_TO_ENDPOINT = $0025;
  URB_FUNCTION_SET_DESCRIPTOR_TO_INTERFACE = $0029;

  URB_FUNCTION_SET_FEATURE_TO_DEVICE = $000D;
  URB_FUNCTION_SET_FEATURE_TO_INTERFACE = $000E;
  URB_FUNCTION_SET_FEATURE_TO_ENDPOINT = $000;
  URB_FUNCTION_SET_FEATURE_TO_OTHER = $0023;

  URB_FUNCTION_CLEAR_FEATURE_TO_DEVICE = $0010;
  URB_FUNCTION_CLEAR_FEATURE_TO_INTERFACE = $0011;
  URB_FUNCTION_CLEAR_FEATURE_TO_ENDPOINT = $0012;
  URB_FUNCTION_CLEAR_FEATURE_TO_OTHER = $0022;

  URB_FUNCTION_GET_STATUS_FROM_DEVICE = $0013;
  URB_FUNCTION_GET_STATUS_FROM_INTERFACE = $0014;
  URB_FUNCTION_GET_STATUS_FROM_ENDPOINT = $0015;
  URB_FUNCTION_GET_STATUS_FROM_OTHER = $0021;

// direction is specified in TransferFlags

  URB_FUNCTION_RESERVED0 = $0016;

//
// These are for sending vendor and class commands
// on the default pipe
//
// direction is specified in TransferFlags
//
  URB_FUNCTION_VENDOR_DEVICE = $0017;
  URB_FUNCTION_VENDOR_INTERFACE = $0018;
  URB_FUNCTION_VENDOR_ENDPOINT = $0019;
  URB_FUNCTION_VENDOR_OTHER = $0020;

  URB_FUNCTION_CLASS_DEVICE = $001A;
  URB_FUNCTION_CLASS_INTERFACE = $001B;
  URB_FUNCTION_CLASS_ENDPOINT = $001C;
  URB_FUNCTION_CLASS_OTHER = $001;

//
// Reserved function codes
//
const
  URB_FUNCTION_RESERVED = $001D;

  URB_FUNCTION_GET_CONFIGURATION = $0026;
  URB_FUNCTION_GET_INTERFACE = $0027;

  URB_FUNCTION_LAST = $0029;

//
// Values for URB TransferFlags Field
//
function USBD_TRANSFER_DIRECTION(x:DWORD):DWORD;

// USBD_TRANSFER_DIRECTION(x) = ((x) & USBD_TRANSFER_DIRECTION_IN);
const
  USBD_TRANSFER_DIRECTION_OUT = 0;

  USBD_TRANSFER_DIRECTION_BIT = 0;
  USBD_TRANSFER_DIRECTION_IN = (1 SHL USBD_TRANSFER_DIRECTION_BIT);

  USBD_SHORT_TRANSFER_OK_BIT = 1;
  USBD_SHORT_TRANSFER_OK = (1 SHL USBD_SHORT_TRANSFER_OK_BIT);

  USBD_START_ISO_TRANSFER_ASAP_BIT = 2;
  USBD_START_ISO_TRANSFER_ASAP = (1 SHL USBD_START_ISO_TRANSFER_ASAP_BIT);

  USBD_ISO_START_FRAME_RANGE = 1024;

type
  USBD_STATUS = LongInt;

//
// USBD status codes
//
// Status values are 32 bit values layed out as follows:
//
// 3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
// 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
// +---+---------------------------+-------------------------------+
// | S | Status Code |
// +---+---------------------------+-------------------------------+
//
// where
//
// S - is the state code
//
// 00 - completed with success
// 01 - request is pending
// 10 - completed with error, endpoint not stalled
// 11 - completed with error, endpoint stalled
//
//
// Code - is the status code
//

//
// Generic test for success on any status value (non-negative numbers
// indicate success).
//

//  USBD_SUCCESS(Status) = ((USBD_STATUS)(Status) >= 0);
function USBD_SUCCESS(Status:Longint):boolean;

//
// Generic test for pending status value.
//

//  USBD_PENDING(Status) = ((ULONG)(Status) >> 30 == 1);
function USBD_PENDING(Status:DWORD):boolean;
//  USBD_PENDING(Status) = ((ULONG)(Status) >> 30 == 1);

//
// Generic test for error on any status value.
//

//  USBD_ERROR(Status) = ((USBD_STATUS)(Status) < 0);
function USBD_ERROR(Status:DWORD):boolean;

//
// Generic test for stall on any status value.
//

//  USBD_HALTED(Status) = ((ULONG)(Status) >> 30 == 3);
function USBD_HALTED(Status:DWORD):boolean;

//
// Macro to check the status code only
//

//  USBD_STATUS(Status) = ((ULONG)(Status) & 0x0FFFFFFFL);
function _USBD_STATUS(Status:DWORD):DWORD;

const
  USBD_STATUS_SUCCESS = ($00000000);
  USBD_STATUS_PENDING = ($40000000);
  USBD_STATUS_HALTED = ($C0000000);
  USBD_STATUS_ERROR = ($80000000);

//
// HC status codes
// Note: these status codes have both the error and the stall bit set.
//
  USBD_STATUS_CRC = $C0000001;
  USBD_STATUS_BTSTUFF = $C0000002;
  USBD_STATUS_DATA_TOGGLE_MISMATCH = ($C0000003);
  USBD_STATUS_STALL_PID = ($C0000004);
  USBD_STATUS_DEV_NOT_RESPONDING = ($C0000005);
  USBD_STATUS_PID_CHECK_FAILURE = ($C0000006);
  USBD_STATUS_UNEXPECTED_PID = ($C0000007);
  USBD_STATUS_DATA_OVERRUN = ($C0000008);
  USBD_STATUS_DATA_UNDERRUN = ($C0000009);
  USBD_STATUS_RESERVED1 = ($C000000A);
  USBD_STATUS_RESERVED2 = ($C000000B);
  USBD_STATUS_BUFFER_OVERRUN = ($C000000C);
  USBD_STATUS_BUFFER_UNDERRUN = ($C000000D);
  USBD_STATUS_NOT_ACCESSED = ($C000000F);
  USBD_STATUS_FIFO = ($C0000010);

//
// returned by HCD if a transfer is submitted to an endpoint that is
// stalled
//
  USBD_STATUS_ENDPOINT_HALTED = ($C0000030);

//
// Software status codes
// Note: the following status codes have only the error bit set
//
  USBD_STATUS_NO_MEMORY = ($80000100);
  USBD_STATUS_INVALID_URB_FUNCTION = ($80000200);
  USBD_STATUS_INVALID_PARAMETER = ($80000300);

//
// returned if client driver attempts to close an endpoint/interface
// or configuration with outstanding transfers.
//
  USBD_STATUS_ERROR_BUSY = ($80000400);
//
// returned by USBD if it cannot complete a URB request, typically this
// will be returned in the URB status field when the Irp is completed
// with a more specific NT error code in the irp.status field.
//
  USBD_STATUS_REQUEST_FAILED = ($80000500);

  USBD_STATUS_INVALID_PIPE_HANDLE = ($80000600);

// returned when there is not enough bandwidth avialable
// to open a requested endpoint
  USBD_STATUS_NO_BANDWIDTH = ($80000700);
//
// generic HC error
//
  USBD_STATUS_INTERNAL_HC_ERROR = ($80000800);
//
// returned when a short packet terminates the transfer
// ie USBD_SHORT_TRANSFER_OK bit not set
//
  USBD_STATUS_ERROR_SHORT_TRANSFER = ($80000900);
//
// returned if the requested start frame is not within
// USBD_ISO_START_FRAME_RANGE of the current USB frame,
// note that the stall bit is set
//
  USBD_STATUS_BAD_START_FRAME = ($C0000A00);
//
// returned by HCD if all packets in an iso transfer complete with an error
//
  USBD_STATUS_ISOCH_REQUEST_FAILED = ($C0000B00);
//
// returned by USBD if the frame length control for a given
// HC is already taken by anothe driver
//
  USBD_STATUS_FRAME_CONTROL_OWNED = ($C0000C00);
//
// returned by USBD if the caller does not own frame length control and
// attempts to release or modify the HC frame length
//
  USBD_STATUS_FRAME_CONTROL_NOT_OWNED = ($C0000D00);

//
// set when a transfers is completed due to an AbortPipe request from
// the client driver
//
// Note: no error or stall bit is set for these status codes
//
  USBD_STATUS_CANCELED = ($00010000);
  USBD_STATUS_CANCELING = ($00020000);

type
  HUSBD_PIPE_HANDLE = PVOID;
  HUSBD_CONFIGURATION_HANDLE = PVOID;
  HUSBD_INTERFACE_HANDLE = PVOID;

//
// Value used to indicate the default max transfer size
//
const
  USBD_DEFAULT_MAXIMUM_TRANSFER_SIZE = $1000;//PAGE_SIZE(WDM.h/x86);


//
// structure returned from USBD_GetVersion function
//

type
  _USBD_VERSION_INFORMATION = packed record
    USBDI_Version: ULONG;
    Supported_USB_Version: ULONG;
  end {_USBD_VERSION_INFORMATION};
  TUSBD_VERSION_INFORMATION = _USBD_VERSION_INFORMATION;
  PUSBD_VERSION_INFORMATION = ^_USBD_VERSION_INFORMATION;

type
  TUSBD_PIPE_TYPE = (
    UsbdPipeTypeControl,
    UsbdPipeTypeIsochronous,
    UsbdPipeTypeBulk,
    UsbdPipeTypeInterrupt  );

type
  _USBD_DEVICE_INFORMATION = packed record
    OffsetNext: ULONG;
    UsbdDeviceHandle: PVOID;
    DeviceDescriptor: TUSB_DEVICE_DESCRIPTOR;
  end {_USBD_DEVICE_INFORMATION};
  TUSBD_DEVICE_INFORMATION = _USBD_DEVICE_INFORMATION;
  PUSBD_DEVICE_INFORMATION = ^_USBD_DEVICE_INFORMATION;

//
// URB request structures
//

//
// USBD pipe information structure, this structure
// is returned for each pipe opened thru an
// SELECT_CONFIGURATION or SELECT_INTERFACE request.
//

type
  _USBD_PIPE_INFORMATION = packed record
//
// OUTPUT
// These fields are filled in by USBD
//
    MaximumPacketSize: USHORT;
    EndpointAddress: UCHAR;
// taken from endpoint descriptor
    Interval: UCHAR;
    PipeType: TUSBD_PIPE_TYPE;
    PipeHandle: HUSBD_PIPE_HANDLE;
//
// INPUT
// These fields are filled in by the client driver
//
    MaximumTransferSize: ULONG;
// in bytes.
    PipeFlags: ULONG;
  end {_USBD_PIPE_INFORMATION};
  USBD_PIPE_INFORMATION = _USBD_PIPE_INFORMATION;
  PUSBD_PIPE_INFORMATION = ^_USBD_PIPE_INFORMATION;

function USBD_PIPE_DIRECTION_IN(pipeInformation:PUSBD_PIPE_INFORMATION):boolean;

//
// values for PipeFlags field in USBD_PIPE_INFORMATION field
//

const
  USBD_PF_CHANGE_MAX_PACKET = $00000001;

//
// USBD interface information structure, this structure
// is returned for each interface opened thru an
// SELECT_CONFIGURATION or SELECT_INTERFACE request.
//

type
  _USBD_INTERFACE_INFORMATION = packed record
    Length: USHORT;
// all pipe information structures that
// follow.
//
// INPUT
//
// Interface number and Alternate setting this
// structure is associated with
//
    InterfaceNumber: UCHAR;
    AlternateSetting: UCHAR;
//
// OUTPUT
// These fields are filled in by USBD
//
    _Class: UCHAR;
    SubClass: UCHAR;
    Protocol: UCHAR;
    Reserved: UCHAR;
    InterfaceHandle: HUSBD_INTERFACE_HANDLE;
    NumberOfPipes: ULONG;
//
// INPUT/OUPUT
// see PIPE_INFORMATION
{$IFDEF OSR21_COMPAT}
// NONE SIZE
//    Pipes: Array[0..-1] of USBD_PIPE_INFORMATION;
{$ELSE}
    Pipes: Array[0..0] of USBD_PIPE_INFORMATION;
{$ENDIF}
  end {_USBD_INTERFACE_INFORMATION};
  TUSBD_INTERFACE_INFORMATION = _USBD_INTERFACE_INFORMATION;
  PUSBD_INTERFACE_INFORMATION = ^_USBD_INTERFACE_INFORMATION;

//
// work space in transfer request provided
// for HCDs
//

type
  _URB_HCD_AREA = packed record
    HcdEndpoint: PVOID;
    HcdIrp: PIRP;
    HcdListEntry: LIST_ENTRY;
    HcdListEntry2: LIST_ENTRY;
    HcdCurrentIoFlushPointer: PVOID;
    HcdExtension: PVOID;
  end {_URB_HCD_AREA};

type
  _URB_HEADER = packed record
//
// Fields filled in by client driver
//
    Length: USHORT;
    _Function: USHORT;
    Status: USBD_STATUS;
//
// Fields used only by USBD
//
    UsbdDeviceHandle: PVOID;
// by USBD
    UsbdFlags: ULONG;
  end {_URB_HEADER};

type
  _URB_SELECT_INTERFACE = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    ConfigurationHandle: HUSBD_CONFIGURATION_HANDLE;
// client must input AlternateSetting & Interface Number
// class driver returns interface and handle
// for new alternate setting
    _Interface: TUSBD_INTERFACE_INFORMATION;
  end {_URB_SELECT_INTERFACE};

type
  _URB_SELECT_CONFIGURATION = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
// NULL indicates to set the device
// to the 'unconfigured' state
// ie set to configuration 0
    ConfigurationDescriptor: PUSB_CONFIGURATION_DESCRIPTOR;
    ConfigurationHandle: HUSBD_CONFIGURATION_HANDLE;
    _Interface: TUSBD_INTERFACE_INFORMATION;
  end {_URB_SELECT_CONFIGURATION};

//
// This structure used for ABORT_PIPE & RESET_PIPE
//

type
  _URB_PIPE_REQUEST = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    PipeHandle: HUSBD_PIPE_HANDLE;
    Reserved: ULONG;
  end {_URB_PIPE_REQUEST};

//
// This structure used for
// TAKE_FRAME_LENGTH_CONTROL &
// RELEASE_FRAME_LENGTH_CONTROL
//

type
  _URB_FRAME_LENGTH_CONTROL = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
  end {_URB_FRAME_LENGTH_CONTROL};

type
  _URB_GET_FRAME_LENGTH = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    FrameLength: ULONG;
    FrameNumber: ULONG;
  end {_URB_GET_FRAME_LENGTH};

type
  _URB_SET_FRAME_LENGTH = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    FrameLengthDelta: LongInt;
  end {_URB_SET_FRAME_LENGTH};

type
  _URB_GET_CURRENT_FRAME_NUMBER = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    FrameNumber: ULONG;
  end {_URB_GET_CURRENT_FRAME_NUMBER};

//
// Structures for specific control transfers
// on the default pipe.
//

// GET_DESCRIPTOR
// SET_DESCRIPTOR

type
  _URB_CONTROL_DESCRIPTOR_REQUEST = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    Reserved: PVOID;
    Reserved0: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
    hca: _URB_HCD_AREA;
    Reserved1: USHORT;
    Index: UCHAR;
    DescriptorType: UCHAR;
    LanguageId: USHORT;
    Reserved2: USHORT;
  end {_URB_CONTROL_DESCRIPTOR_REQUEST};

// GET_STATUS

type
  _URB_CONTROL_GET_STATUS_REQUEST = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    Reserved: PVOID;
    Reserved0: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
    Reserved1: Array[0..4-1] of UCHAR;
    Index: USHORT;
    Reserved2: USHORT;
  end {_URB_CONTROL_GET_STATUS_REQUEST};

// SET_FEATURE
// CLEAR_FEATURE

type
  _URB_CONTROL_FEATURE_REQUEST = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    Reserved: Array[0..20-1] of UCHAR;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
    Reserved0: USHORT;
    FeatureSelector: USHORT;
    Index: USHORT;
    Reserved1: USHORT;
  end {_URB_CONTROL_FEATURE_REQUEST};

// VENDOR & CLASS

type
  _URB_CONTROL_VENDOR_OR_CLASS_REQUEST = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    Reserved: PVOID;
    TransferFlags: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
    RequestTypeReservedBits: UCHAR;
    Request: UCHAR;
    Value: USHORT;
    Index: USHORT;
    Reserved1: USHORT;
  end {_URB_CONTROL_VENDOR_OR_CLASS_REQUEST};


type
  _URB_CONTROL_GET_INTERFACE_REQUEST = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    Reserved: PVOID;
    Reserved0: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
    Reserved1: Array[0..4-1] of UCHAR;
    _Interface: USHORT;
    Reserved2: USHORT;
  end {_URB_CONTROL_GET_INTERFACE_REQUEST};


type
  _URB_CONTROL_GET_CONFIGURATION_REQUEST = packed record
{$IFDEF OSR21_COMPAT}
    Hdr: _URB_HEADER;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    Reserved: PVOID;
    Reserved0: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
    Reserved1: Array[0..8-1] of UCHAR;
  end {_URB_CONTROL_GET_CONFIGURATION_REQUEST};


//
// request format for a control transfer on
// the non-default pipe.
//

type
  _URB_CONTROL_TRANSFER = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    PipeHandle: HUSBD_PIPE_HANDLE;
    TransferFlags: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
    SetupPacket: Array[0..8-1] of UCHAR;
  end {_URB_CONTROL_TRANSFER};


type
  _URB_BULK_OR_INTERRUPT_TRANSFER = packed record
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    PipeHandle: HUSBD_PIPE_HANDLE;
    TransferFlags: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
  end {_URB_BULK_OR_INTERRUPT_TRANSFER};


//
// ISO Transfer request
//
// TransferBufferMDL must point to a single virtually
// contiguous buffer.
//
// StartFrame - the frame to send/receive the first packet of
// the request.
//
// NumberOfPackets - number of packets to send in this request
//
//
// IsoPacket Array
//
// Input: Offset - offset of the packet from the beginig
// of the client buffer.
// Output: Length - is set to the actual length of the packet
// (For IN transfers).
// Status: error that occurred during transmission or
// reception of the packet.
//

type
  _USBD_ISO_PACKET_DESCRIPTOR = packed record
    Offset: ULONG;
// buffer.

    Length: ULONG;
// OUTPUT 0 for OUT.
    Status: USBD_STATUS;
  end {_USBD_ISO_PACKET_DESCRIPTOR};
  USBD_ISO_PACKET_DESCRIPTOR = _USBD_ISO_PACKET_DESCRIPTOR;
  PUSBD_ISO_PACKET_DESCRIPTOR = ^_USBD_ISO_PACKET_DESCRIPTOR;

type
  _URB_ISOCH_TRANSFER = packed record
//
// This block is the same as CommonTransfer
//
{$IFDEF OSR21_COMPAT}
    _URB_HEADER: Integer;
{$ELSE}
    Hdr: _URB_HEADER;
{$ENDIF}
    PipeHandle: HUSBD_PIPE_HANDLE;
    TransferFlags: ULONG;
    TransferBufferLength: ULONG;
    TransferBuffer: PVOID;
    TransferBufferMDL: PMDL;
    UrbLink: Pointer;
// if this is a chain of commands
    hca: _URB_HCD_AREA;
//
// this block contains transfer fields
// specific to isochronous transfers
//

// 32 bit frame number to begin this transfer on, must be within 1000
// frames of the current USB frame or an error is returned.

// START_ISO_TRANSFER_ASAP flag in transferFlags:
// If this flag is set and no transfers have been submitted
// for the pipe then the transfer will begin on the next frame
// and StartFrame will be updated with the frame number the transfer
// was started on.
// If this flag is set and the pipe has active transfers then
// the transfer will be queued to begin on the frame after the
// last transfer queued is completed.
//
    StartFrame: ULONG;
// number of packets that make up this request
    NumberOfPackets: ULONG;
// number of packets that completed with errors
    ErrorCount: ULONG;
{$IFDEF OSR21_COMPAT}
//  IsoPacket: Array[0..0-1] of USBD_ISO_PACKET_DESCRIPTOR;
{$ELSE}
    IsoPacket: Array[0..1-1] of USBD_ISO_PACKET_DESCRIPTOR;
{$ENDIF}
  end {_URB_ISOCH_TRANSFER};


type
  _URB = record case integer of
     0: ( UrbHeader: _URB_HEADER );
     1: ( UrbSelectInterface: _URB_SELECT_INTERFACE );
     2: ( UrbSelectConfiguration: _URB_SELECT_CONFIGURATION );
     3: ( UrbPipeRequest: _URB_PIPE_REQUEST );
     4: ( UrbFrameLengthControl: _URB_FRAME_LENGTH_CONTROL );
     5: ( UrbGetFrameLength: _URB_GET_FRAME_LENGTH );
     6: ( UrbSetFrameLength: _URB_SET_FRAME_LENGTH );
     7: ( UrbGetCurrentFrameNumber: _URB_GET_CURRENT_FRAME_NUMBER );
     8: ( UrbControlTransfer: _URB_CONTROL_TRANSFER );
     9: ( UrbBulkOrInterruptTransfer: _URB_BULK_OR_INTERRUPT_TRANSFER );
    10:( UrbIsochronousTransfer: _URB_ISOCH_TRANSFER );
// for standard control transfers on the default pipe
    11: ( UrbControlDescriptorRequest: _URB_CONTROL_DESCRIPTOR_REQUEST );
    12: ( UrbControlGetStatusRequest: _URB_CONTROL_GET_STATUS_REQUEST );
    13: ( UrbControlFeatureRequest: _URB_CONTROL_FEATURE_REQUEST );
    14: ( UrbControlVendorClassRequest: _URB_CONTROL_VENDOR_OR_CLASS_REQUEST );
    15: ( UrbControlGetInterfaceRequest: _URB_CONTROL_GET_INTERFACE_REQUEST );
    16:( UrbControlGetConfigurationRequest: _URB_CONTROL_GET_CONFIGURATION_REQUEST );
  end {_URB};
  TURB = _URB;
  PURB = ^_URB;

implementation

function USBD_TRANSFER_DIRECTION(x:DWORD):DWORD;
begin
  Result:= ((x) and USBD_TRANSFER_DIRECTION_IN);
end;

function USBD_SUCCESS(Status:Longint):boolean;
begin
  Result:= (Status >= 0);
end;

function USBD_PENDING(Status:DWORD):boolean;
begin
  Result:= (ULONG(Status) SHR 30) = 1;
end;

function USBD_ERROR(Status:DWORD):boolean;
begin
  RESULT:= (USBD_STATUS(Status) < 0);
end;

function USBD_HALTED(Status:DWORD):boolean;
begin
  Result:= (ULONG(Status) SHR 30) = 3;
end;

function _USBD_STATUS(Status:DWORD):DWORD;
begin
  Result:= (ULONG(Status) and $0FFFFFFF);
end;


function USBD_PIPE_DIRECTION_IN(pipeInformation:PUSBD_PIPE_INFORMATION):boolean;
begin
  Result:=(pipeInformation^.EndpointAddress and USB_ENDPOINT_DIRECTION_MASK)<>0;
end;

end.
