unit USBIOCTL;

interface
uses
  Windows,ActiveX,usb100;

const
  GUID_CLASS_USBHUB : TGUID = (D1:$F18A0E88;D2:$C30C;D3:$11D0;D4:($88,$15,$00,$a0,$C9,$06,$BE,$D8));
//f18a0e88-c30c-11d0-8815-00a0c906bed8

// BUGBUG we need a device type added to DEVIOCTL.H
const
  FILE_DEVICE_UNKNOWN = $00000022;
  FILE_DEVICE_USB = FILE_DEVICE_UNKNOWN;

  FILE_ANY_ACCESS   =  0;
  FILE_READ_ACCESS  =  1;    // file & pipe
  FILE_WRITE_ACCESS =  2;    // file & pipe

  METHOD_BUFFERED   =  0;
  METHOD_IN_DIRECT  =  1;
  METHOD_OUT_DIRECT =  2;
  METHOD_NEITHER    =  3;

//
// USB IOCTLS
//

const
  USB_IOCTL_INTERNAL_INDEX = $0000;
  USB_IOCTL_INDEX = $00ff;

//
// USB Internal IOCtls
//

// -This IOCTL is used by client drivers to submit URB (USB Request Blocks)
// -Parameters.Others.Argument1 = pointer to URB
var
  IOCTL_INTERNAL_USB_SUBMIT_URB : DWORD;
  IOCTL_INTERNAL_USB_RESET_PORT	: DWORD;

// -This IOCTL is used internally by the hub driver

// -Parameters.Others.Argument1 = pointer to be filled in with RootHubPdo;
// -Parameters.Others.Argument2 = pointer to be filled in with HcdDeviceObject;

  IOCTL_INTERNAL_USB_GET_ROOTHUB_PDO	: DWORD;


// IOCTL_INTERNAL_USB_GET_PORT_STATUS

// -This IOCTL returns the current port status
// -Parameters.Others.Argument1 = pointer to port status register (ULONG)
// -status bits are:
// -USBD_PORT_ENABLED

const
  USBD_PORT_ENABLED = $00000001;
  USBD_PORT_CONNECTED = $00000002;

var
  IOCTL_INTERNAL_USB_GET_PORT_STATUS : DWORD;

// -This IOCTL will request the hub to re-enable a disabled port
  IOCTL_INTERNAL_USB_ENABLE_PORT : DWORD;

// -This IOCTL is used internally by the hub driver
// -Parameters.Others.Argument1 = pointer to be count of hubs in chain;
  IOCTL_INTERNAL_USB_GET_HUB_COUNT : DWORD;

// -This will simulate a plug/unplug on the port
  IOCTL_INTERNAL_USB_CYCLE_PORT : DWORD;

//
// USB Public IOCtls
//

//
// this ioctl is for adding debug hooks to HCDs
//
  IOCTL_USB_HCD_GET_STATS : DWORD;

//
// These ioctls are used for USB diagnostic and test applications
//
  IOCTL_USB_DIAGNOSTIC_MODE_ON,
  IOCTL_USB_DIAGNOSTIC_MODE_OFF,
  IOCTL_USB_GET_ROOT_HUB_NAME : DWORD;

// -Utility IOCTLS supported by the hub device

//
// These ioctls are supported by the hub driver for
// use by user mode USB utilities.
//
  IOCTL_USB_GET_NODE_INFORMATION,
  IOCTL_USB_GET_NODE_CONNECTION_INFORMATION,
  IOCTL_USB_GET_DESCRIPTOR_FROM_NODE_CONNECTION,
  IOCTL_USB_GET_NODE_CONNECTION_NAME,
  IOCTL_USB_DIAG_IGNORE_HUBS_ON,
  IOCTL_USB_DIAG_IGNORE_HUBS_OFF,
  IOCTL_USB_GET_NODE_CONNECTION_DRIVERKEY_NAME,
  IOCTL_GET_HCD_DRIVERKEY_NAME : DWORD;

// structures for user mode ioctls
//

type
  TUSB_HUB_NODE = (
    UsbHub,
    UsbMIParent  );

type
  {$EXTERNALSYM _USB_HUB_INFORMATION}
  _USB_HUB_INFORMATION = packed record
//
// copy of data from hub descriptor
//
    HubDescriptor: TUSB_HUB_DESCRIPTOR;
    HubIsBusPowered: BOOLEAN;
  end {_USB_HUB_INFORMATION};
  TUSB_HUB_INFORMATION = _USB_HUB_INFORMATION;
  PUSB_HUB_INFORMATION = ^_USB_HUB_INFORMATION;

type
  {$EXTERNALSYM _USB_MI_PARENT_INFORMATION}
  _USB_MI_PARENT_INFORMATION = packed record
    NumberOfInterfaces: ULONG;
  end {_USB_MI_PARENT_INFORMATION};
  TUSB_MI_PARENT_INFORMATION = _USB_MI_PARENT_INFORMATION;
  PUSB_MI_PARENT_INFORMATION = ^_USB_MI_PARENT_INFORMATION;

type
  {$EXTERNALSYM _u}
  _u = record case integer of
    0 : ( HubInformation: TUSB_HUB_INFORMATION );
    1 : ( MiParentInformation: TUSB_MI_PARENT_INFORMATION );
  end;
  {$EXTERNALSYM _USB_NODE_INFORMATION}
  _USB_NODE_INFORMATION = packed record
    NodeType: TUSB_HUB_NODE;
    u : _u;
  end {_USB_NODE_INFORMATION};
  TUSB_NODE_INFORMATION = _USB_NODE_INFORMATION;
  PUSB_NODE_INFORMATION = ^_USB_NODE_INFORMATION;

type
  {$EXTERNALSYM _USB_PIPE_INFO}
  _USB_PIPE_INFO = packed record
    EndpointDescriptor: TUSB_ENDPOINT_DESCRIPTOR;
  end {_USB_PIPE_INFO};
  TUSB_PIPE_INFO = _USB_PIPE_INFO;
  PUSB_PIPE_INFO = ^_USB_PIPE_INFO;

//
// BUGBUG
// more status codes?
//

type
  TUSB_CONNECTION_STATUS = (
    NoDeviceConnected,
    DeviceConnected,
    // failure codes, these map to fail reasons
    DeviceFailedEnumeration,
    DeviceGeneralFailure,
    DeviceCausedOvercurrent,
    DeviceNotEnougshPower
  );
  PUSB_CONNECTION_STATUS = ^TUSB_CONNECTION_STATUS;

type
  {$EXTERNALSYM _USB_NODE_CONNECTION_INFORMATION}
  _USB_NODE_CONNECTION_INFORMATION = packed record
    ConnectionIndex: ULONG;
// usb device descriptor returned by this device
// during enumeration
    DeviceDescriptor: TUSB_DEVICE_DESCRIPTOR;
    CurrentConfigurationValue: UCHAR;
    LowSpeed: BOOLEAN;
    DeviceIsHub: BOOLEAN;
    DeviceAddress: USHORT;
    NumberOfOpenPipes: ULONG;
    ConnectionStatus: TUSB_CONNECTION_STATUS;
    // NONE.
//    PipeList: Array[0..0] of USB_PIPE_INFO;
  end {_USB_NODE_CONNECTION_INFORMATION};
  TUSB_NODE_CONNECTION_INFORMATION = _USB_NODE_CONNECTION_INFORMATION;
  PUSB_NODE_CONNECTION_INFORMATION = ^_USB_NODE_CONNECTION_INFORMATION;

type
  {$EXTERNALSYM _USB_NODE_CONNECTION_DRIVERKEY_NAME}
  _USB_NODE_CONNECTION_DRIVERKEY_NAME = packed record
    ConnectionIndex: ULONG;
    ActualLength: ULONG;
// unicode name for the devnode.
    DriverKeyName: Array[0..0] of WCHAR;
  end {_USB_NODE_CONNECTION_DRIVERKEY_NAME};
  TUSB_NODE_CONNECTION_DRIVERKEY_NAME = _USB_NODE_CONNECTION_DRIVERKEY_NAME;
  PUSB_NODE_CONNECTION_DRIVERKEY_NAME = ^_USB_NODE_CONNECTION_DRIVERKEY_NAME;

type
  {$EXTERNALSYM _USB_NODE_CONNECTION_NAME}
  _USB_NODE_CONNECTION_NAME = packed record
    ConnectionIndex: ULONG;
    ActualLength: ULONG;
// unicode symbolic name for this node if it is a hub or parent driver
// null if this node is a device.
    NodeName: Array[0..1-1] of WCHAR;
  end {_USB_NODE_CONNECTION_NAME};
  TUSB_NODE_CONNECTION_NAME = _USB_NODE_CONNECTION_NAME;
  PUSB_NODE_CONNECTION_NAME = ^_USB_NODE_CONNECTION_NAME;

type
  {$EXTERNALSYM _USB_ROOT_HUB_NAME}
  _USB_ROOT_HUB_NAME = packed record
    ActualLength: ULONG;
// NULL terminated unicode symbolic name for the root hub
    RootHubName: Array[0..1-1] of WCHAR;
  end {_USB_ROOT_HUB_NAME};
  TUSB_ROOT_HUB_NAME = _USB_ROOT_HUB_NAME;
  PUSB_ROOT_HUB_NAME = ^_USB_ROOT_HUB_NAME;

type
  {$EXTERNALSYM _USB_HCD_DRIVERKEY_NAME}
  _USB_HCD_DRIVERKEY_NAME = packed record
    ActualLength: ULONG;
// NULL terminated unicode driverkeyname for hcd
    DriverKeyName: Array[0..1-1] of WCHAR;
  end {_USB_HCD_DRIVERKEY_NAME};
  TUSB_HCD_DRIVERKEY_NAME = _USB_HCD_DRIVERKEY_NAME;
  PUSB_HCD_DRIVERKEY_NAME = ^_USB_HCD_DRIVERKEY_NAME;

type
  {$EXTERNALSYM _SetupPacket}
  _SetupPacket = packed record
    bmRequest: UCHAR;
    bRequest: UCHAR;
    wValue: USHORT;
    wIndex: USHORT;
    wLength: USHORT;
  end;
  {$EXTERNALSYM _USB_DESCRIPTOR_REQUEST}
  _USB_DESCRIPTOR_REQUEST = packed record
    ConnectionIndex: ULONG;
    SetupPacket : _SetupPacket;
    // UCHAR Data[0];
  end; {_USB_DESCRIPTOR_REQUEST}
  TUSB_DESCRIPTOR_REQUEST = _USB_DESCRIPTOR_REQUEST;
  PUSB_DESCRIPTOR_REQUEST = ^_USB_DESCRIPTOR_REQUEST;

//
// Structure for returning HCD debug and statistic information to
// a user mode application.
//

type
  {$EXTERNALSYM _HCD_STAT_COUNTERS}
  _HCD_STAT_COUNTERS = packed record
    BytesTransferred: ULONG;
    IsoMissedCount: USHORT;
    DataOverrunErrorCount: USHORT;
    CrcErrorCount: USHORT;
    ScheduleOverrunCount: USHORT;
    TimeoutErrorCount: USHORT;
    InternalHcErrorCount: USHORT;
    BufferOverrunErrorCount: USHORT;
    SWErrorCount: USHORT;
    StallPidCount: USHORT;
    PortDisableCount: USHORT;
  end {_HCD_STAT_COUNTERS};
  THCD_STAT_COUNTERS = _HCD_STAT_COUNTERS;
  PHCD_STAT_COUNTERS = ^_HCD_STAT_COUNTERS;


type
  {$EXTERNALSYM _HCD_STAT_INFORMATION}
  _HCD_STAT_INFORMATION = packed record
    Reserved1: ULONG;
    Reserved2: ULONG;
    ResetCounters: ULONG;
    TimeRead: LARGE_INTEGER;
//
// stat registers
//
    Counters: THCD_STAT_COUNTERS;
  end {_HCD_STAT_INFORMATION};
  THCD_STAT_INFORMATION = _HCD_STAT_INFORMATION;
  PHCD_STAT_INFORMATION = ^_HCD_STAT_INFORMATION;

// ntddk.h defined macro.
function CTL_CODE( _DeviceType, _Function, _Method, _Access:DWORD ):DWORD;

implementation

function CTL_CODE( _DeviceType, _Function, _Method, _Access:DWORD ):DWORD;
begin
  Result:= (_DeviceType SHL 16) or (_Access SHL 14) or (_Function SHL 2) or _Method;
end;

initialization

IOCTL_INTERNAL_USB_SUBMIT_URB:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX,METHOD_NEITHER,FILE_ANY_ACCESS);
IOCTL_INTERNAL_USB_RESET_PORT:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+1,METHOD_NEITHER,FILE_ANY_ACCESS);
IOCTL_INTERNAL_USB_GET_ROOTHUB_PDO:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+3,METHOD_NEITHER,FILE_ANY_ACCESS);

IOCTL_INTERNAL_USB_GET_PORT_STATUS:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+4,METHOD_NEITHER,FILE_ANY_ACCESS);
IOCTL_INTERNAL_USB_ENABLE_PORT:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+5,METHOD_NEITHER,FILE_ANY_ACCESS);
IOCTL_INTERNAL_USB_GET_HUB_COUNT:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+6,METHOD_NEITHER,FILE_ANY_ACCESS);
IOCTL_INTERNAL_USB_CYCLE_PORT:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+7,METHOD_NEITHER,FILE_ANY_ACCESS);
IOCTL_USB_HCD_GET_STATS:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+8,METHOD_NEITHER,FILE_ANY_ACCESS);
IOCTL_USB_DIAGNOSTIC_MODE_ON:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INTERNAL_INDEX+9,METHOD_NEITHER,FILE_ANY_ACCESS);

IOCTL_USB_DIAGNOSTIC_MODE_OFF:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+2,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_USB_GET_ROOT_HUB_NAME:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+3,METHOD_BUFFERED,FILE_ANY_ACCESS);

IOCTL_USB_GET_NODE_INFORMATION:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+3,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_USB_GET_NODE_CONNECTION_INFORMATION :=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+4,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_USB_GET_DESCRIPTOR_FROM_NODE_CONNECTION:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+5,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_USB_GET_NODE_CONNECTION_NAME:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+6,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_USB_DIAG_IGNORE_HUBS_ON:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+7,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_USB_DIAG_IGNORE_HUBS_OFF:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+8,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_USB_GET_NODE_CONNECTION_DRIVERKEY_NAME:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+9,METHOD_BUFFERED,FILE_ANY_ACCESS);
IOCTL_GET_HCD_DRIVERKEY_NAME:=
  CTL_CODE(FILE_DEVICE_USB,USB_IOCTL_INDEX+10,METHOD_BUFFERED,FILE_ANY_ACCESS);

end.
